﻿
' --------------------------------------------------------------------------------------------
'  REFERENCES
' --------------------------------------------------------------------------------------------
' - Nuget WebView2
' - WebView2 SDK
' - Newtonsoft.Json
' - Update Nuget Package - https://www.nuget.org/packages/System.Threading.Tasks.Extensions
' --------------------------------------------------------------------------------------------

Imports System.Runtime.InteropServices

Public Class Form1

    ' ===================================================================================================
    '  OPEN CLOSE FORM
    ' ===================================================================================================
    Private Sub Form1_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load
        '
        ' ------------------------------------------- if the same EXE file is already running then close
        If Not KeyPressedNow(Keys.ShiftKey) Then
            If MyExecFileIsAlreadyRunning(Application.ExecutablePath) Then
                Beep()
                End
            End If
        End If
        '
        ShowWindowTitle()
        InitPanels()
        Buttons_Create()
        SetInitialButtonNamesColorsAndPositions()
        ResetButtonIdentifiersAndSlots()
        Load_INI()
        ' ------------------------------------------------- 
        SplitContainer1.Panel1.BorderStyle = BorderStyle.Fixed3D
        SplitContainer1.Panel2.BorderStyle = BorderStyle.Fixed3D
        WV2.Location = New Point(4, 4)
        WV2.Width = SplitContainer1.Panel1.ClientSize.Width - 8
        WV2.Height = SplitContainer1.Panel1.ClientSize.Height - 8
        PrintTextBox.Location = New Point(2, 2)
        PrintTextBox.Width = SplitContainer1.Panel2.ClientSize.Width - 6
        PrintTextBox.Height = SplitContainer1.Panel2.ClientSize.Height - 6
        ' ------------------------------------------------- 
        Load_Program()
        UpdateStartStopButtons()
        ShowHideTextAndTools()
        ReadBloclyToolboxFromFile()
        WebBrowser_Start()
        UpdateAllPanels()
        Set_ZOOM()
        ToolTips_Init()
        Timer20ms.Interval = 20
        Timer20ms.Start()
        Timer50ms.Interval = 50
        Timer50ms.Start()
        EventsAreEnabled = True
        InitializeButtons = False
        ' ------------------------------------------------- this makes SendKeys faster
        SchedulerMaxPrecision()
        SetFormOpacity()
    End Sub

    Private Sub Form1_FormClosing(ByVal sender As Object, ByVal e As System.Windows.Forms.FormClosingEventArgs) Handles Me.FormClosing
        Timer20ms.Stop()
        Timer50ms.Stop()
        Save_INI()
        StopRunning()
        ExecutionLoopEnabled = False
        Save_Program(ProgramFile_PathAndName)
        SchedulerDefaultPrecision()
        COM_Close()
        If Not Debugger.IsAttached Then
            CloseAllOpenedProcesses()
        End If
    End Sub

    Private Sub Form1_LocationChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.LocationChanged
        If Not EventsAreEnabled Then Return
        LimitFormPosition(Me)
    End Sub

    Private Sub Form1_Resize(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Resize
        If Not EventsAreEnabled Then Return
        LimitFormPosition(Me)
        ShowHideTextAndTools()
        UpdateAllPanels()
        ' ------------------------------------------------------------------ Set Visible Buttons (after EventsAreEnabled)
        SetVisibleButtons()
    End Sub

    Friend Sub ShowWindowTitle()
        If ProgramFile_PathAndName <> "" Then
            Text = AppTitleAndVersion() + "  -  Program :  " + IO.Path.GetFileName(ProgramFile_PathAndName)
        Else
            Text = AppTitleAndVersion()
        End If
    End Sub

    Friend Sub ShowProgramTitle()
        ShowHideTextAndTools()
    End Sub

    ' ===================================================================================================
    '  SPLIT CONTAINER - Refresh controls while moving
    ' ===================================================================================================
    Private Sub SplitContainer1_MouseDown(sender As Object, e As MouseEventArgs) Handles SplitContainer1.MouseDown
        SplitContainer1.IsSplitterFixed = True
    End Sub
    Private Sub SplitContainer1_MouseUp(sender As Object, e As MouseEventArgs) Handles SplitContainer1.MouseUp
        SplitContainer1.IsSplitterFixed = False
    End Sub
    Private Sub SplitContainer1_MouseMove(sender As Object, e As MouseEventArgs) Handles SplitContainer1.MouseMove
        If e.Button = MouseButtons.Left Then
            If e.Y > 0 And e.Y < SplitContainer1.Height Then
                SplitContainer1.SplitterDistance = e.Y
                SplitContainer1.Refresh()
            End If
        End If
    End Sub

    ' ===================================================================================================
    '  UNDO / REDO / LOAD / SAVE
    ' ===================================================================================================
    Private Sub Chk_Undo_Click(sender As Object, e As EventArgs) Handles Chk_ControlsUndo.Click
        WV2.Select() : WV2.Focus()
        SendKeys.Send("^z")
    End Sub
    Private Sub Chk_Redo_Click(sender As Object, e As EventArgs) Handles Chk_ControlsRedo.Click
        WV2.Select() : WV2.Focus()
        SendKeys.Send("^y")
    End Sub
    Private Sub Chk_Load_MouseDown(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles Chk_ControlsLoad.MouseDown
        Chk_ControlsLoad.Checked = True
        If Running Then StopRunning()
        If e.Button = Windows.Forms.MouseButtons.Left Then
            LoadProgramDialog()
        Else
            Save_Program(ProgramFile_PathAndName)
            Load_Program(ProgramFile_PathAndName)
        End If
        Save_INI()
        Chk_ControlsLoad.Checked = False
    End Sub
    Private Sub Chk_Save_MouseDown(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles Chk_ControlsSave.MouseDown
        Chk_ControlsSave.Checked = True
        If Running Then StopRunning()
        If e.Button = Windows.Forms.MouseButtons.Left Then
            SaveProgramDialog()
        Else
            Save_Program(ProgramFile_PathAndName)
        End If
        Save_INI()
        Chk_ControlsSave.Checked = False
    End Sub

    ' ===================================================================================================
    '  RUN / STEP
    ' ===================================================================================================
    Private Sub Chk_Run_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Chk_ControlsRun.CheckedChanged
        If Not EventsAreEnabled Then Return
        EnableAllButtons()
        Running = Not Running
        If Not Running Then
            AddTextToOutputBox("RUN PAUSED")
            Chk_Continue.BackColor = Color.FromArgb(240, 240, 80)
        End If
        UpdateStartStopButtons()
        StartStopRunning()
    End Sub

    Friend Sub UpdateStartStopButtons()
        Dim old As Boolean = EventsAreEnabled
        EventsAreEnabled = False
        If Running Then
            Chk_ControlsRun.Checked = False
            Chk_ControlsRun.Text = "STOP"
            Chk_ControlsUndo.Visible = False
            Chk_ControlsRedo.Visible = False
            Chk_ControlsLoad.Visible = False
            Chk_ControlsSave.Visible = False
            Chk_Continue.Checked = False
            Chk_Continue.Text = "STOP"
        Else
            Chk_ControlsRun.Checked = True
            Chk_ControlsRun.Text = "RUN"
            Chk_ControlsUndo.Visible = True
            Chk_ControlsRedo.Visible = True
            Chk_ControlsLoad.Visible = True
            Chk_ControlsSave.Visible = True
            Chk_Continue.Checked = True
            Chk_Continue.Text = "CONTINUE"
        End If
        EventsAreEnabled = old
    End Sub

    Friend Sub StartStopRunning(Optional SaveProgram As Boolean = True)
        If Running Then
            ' --------------------------------------------------------
            If SaveProgram Then
                Save_Program(ProgramFile_PathAndName)
            End If
            ' --------------------------------------------------------
            ClearTheOpenedProcessesList()
            ' --------------------------------------------------------
            txt_Debug.Text = ""
            ReceivedText = ""
            AddTextToOutputBox("PROGRAM_START")
            UpdateOutputBox()
            Set_ZOOM()
            SetVisibleButtons()
            ' --------------------------------------------------- 
            ExecuteScriptFunction("initInterpreter()")
            ' -------------------------------------------------------- Start running time 
            StartRunningTime()

            ' TODO - NO MAINCODE
            ' -------------------------------------------------------- If NO MAINCODE then execute the first BUTTON
            If Not MainCodePresent() Then
                InitializeButtonUsingLastWorkspace()
                Running = False
                If ExecuteFirstButton() Then
                    Return
                End If
            End If
            ' -------------------------------------------------------- Else start the execution loop
            ExecutionLoop()
        Else
            StopRunning()
        End If
    End Sub

    Private Sub StartExecution()
        If Running Then Return
        Running = True
        StartStopRunning()
    End Sub

    Private Sub StopExecution()
        If Not Running Then Return
        StopRunning(True)
    End Sub

    Private Sub PauseExecution()
        If Not Running Then Return
        StopRunning(False)
    End Sub

    Private Sub StepExecution()
        If Running Then StopRunning(False)
        RunSingleStep()
        Chk_Continue.BackColor = Color.FromArgb(240, 240, 80)
    End Sub

    Private Sub ContinueExecution()
        If Running Then Return
        Running = True
        UpdateStartStopButtons()
        AddTextToOutputBox("PROGRAM_CONTINUED")
        StartRunningTime()
        ExecutionLoop()
    End Sub


    ' ===================================================================================================
    '   VARIABLES AND DEBUG
    ' ===================================================================================================
    Friend Async Sub ShowVariableList()
        If NavigationCompleted Then
            Dim s As String = Await ExecuteScriptFunctionAsync("listVars", New String() {})
            If s <> "" Then
                txt_Debug.SaveScroll()
                txt_Debug.Text = s
                txt_Debug.RestoreScroll()
            End If
        End If
    End Sub

    Private Sub Chk_Continue_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Chk_Continue.Click
        If Not ExecutionLoopEnabled Then
            ContinueExecution()
        Else
            PauseExecution()
        End If
    End Sub

    Private Sub Chk_Step_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Chk_Step.Click
        StepExecution()
    End Sub

    ' ===================================================================================================
    '   TOOLS
    ' ===================================================================================================
    Private Sub Btn_Help_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Btn_Help.Click
        Process.Start(Application.StartupPath + "\Docs")
    End Sub

    Private Sub Chk_ViewCode_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Chk_ViewCode.Click
        ShowBlocklyCode()
    End Sub

    Private Sub Chk_ViewWorkspace_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Chk_ViewWorkspace.Click
        ShowBlocklyWorkspace()
    End Sub

    Private Sub Txt_SlotCommands_TextChanged(sender As Object, e As EventArgs) Handles txt_SlotCommands.TextChanged
        SlotText_Commands = txt_SlotCommands.NumericValueInteger
    End Sub


    ' ===================================================================================================
    '  BUTTONS NOT OPENING menustrip
    ' ===================================================================================================
    Private Sub Chk_Controls_MouseUp(sender As Object, e As MouseEventArgs) Handles Chk_ControlsLoad.MouseUp,
                                                                                    Chk_ControlsSave.MouseUp
        MenuStrip_Exit.Close()
    End Sub

    ' ===================================================================================================
    '  WEB BROWSER
    ' ===================================================================================================
    Friend Async Sub WebBrowser_Start()
        Try
            Await WV2.EnsureCoreWebView2Async()
            WV2.CoreWebView2.Settings.IsScriptEnabled = True
            WV2.Source = New Uri(BlocklyFilesPath + "HTML\Blockly.html")
        Catch
        End Try
    End Sub

    Private NavigationCompleted As Boolean = False
    Private Sub WV2_NavigationCompleted(sender As Object,
                                              e As CoreWebView2NavigationCompletedEventArgs) _
                                                                 Handles WV2.NavigationCompleted
        SetBlocklyToolboxAndWorkspace()
        Set_ZOOM()
        SetSpeed(tkbar_Speed.Value)
        ' --------------------------------------------------------------------- 
        UpdateStartStopButtons()
        ' --------------------------------------------------------------------- 
        WV2.CoreWebView2.AddHostObjectToScript("dotnet", New DomManipulationDotnetInterop)
        ' ---------------------------------------------------------------------
        InitializeButtons = True
        NavigationCompleted = True
        ' --------------------------------------------------------------------- RUN AUTOSTART
        RunAutostart = True
    End Sub

    ' ===================================================================================================
    '  JAVASCRIPT CALLBACKS
    ' ===================================================================================================
    Private Sub WV2_WebMessageReceived(sender As Object,
                                       e As CoreWebView2WebMessageReceivedEventArgs) Handles WV2.WebMessageReceived

        Dim s As String = e.TryGetWebMessageAsString
        Select Case s
            Case "block_changed"
                InitializeButtons = True
            Case ""
        End Select
    End Sub


    ' ===================================================================================================
    '  TIMER 1 --- UPDATE RECEIVED TEXT / RUN AUTOSTART / INIT BUTTONS
    ' ===================================================================================================
    Friend FlagCloseAll As Boolean = False
    Private RunAutostart As Boolean = False
    Private InitializeButtons As Boolean = False

    Private Sub Timer20ms_Tick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Timer20ms.Tick
        '
        If Not NavigationCompleted Then Return

        Timer20ms.Stop()
        ' --------------------------------------------------------------------- COMMANDS VIA SLOT-TEXT
        ' TestCommandSlot()
        ' --------------------------------------------------------------------- UPDATE txt_Output
        If ReceivedTextChanged Then
            ReceivedTextChanged = False
            UpdateOutputBox()
        End If
        ' --------------------------------------------------------------------- RUN AUTOSTART
        If RunAutostart Then
            RunAutostart = False
            StartStopRunning(False)
        End If
        ' --------------------------------------------------------------------- INIT BUTTONS
        If InitializeButtons Then
            InitializeButtons = False
            InitializeButtonUsingLastWorkspace()
        End If
        ' --------------------------------------------------------------------- UPDATE Variable List
        If Running AndAlso Me.WindowState <> FormWindowState.Minimized _
                                              AndAlso GroupBox2.Height > 30 Then
            ShowVariableList()
        End If
        ' --------------------------------------------------------------------- CLOSE ALL
        If FlagCloseAll Then
            CloseAllOpenedProcesses()
            StopRunning()
            Close()
        End If

        Timer20ms.Start()
    End Sub

    Private Sub UpdateOutputBox()
        PrintTextBox.Text = ReceivedText
        PrintTextBox.SelectionStart = 9999
        PrintTextBox.ScrollToCaret()
        PrintTextBox.Refresh()
    End Sub

    Private Sub Timer50ms_Tick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Timer50ms.Tick
        If Not NavigationCompleted Then Return
        If Running Then
            ExecButtonsFromSlots()
            ExecFlashingButtons()
        End If
        ExecCommandsFromTextSlot()
    End Sub

    ' ===================================================================================================
    '  RUNNING START STOP
    ' ===================================================================================================
    Friend Sub StopRunning(Optional ResetHilight As Boolean = True)
        If ExecutionLoopEnabled Then
            ExecutionLoopEnabled = False
        Else
            StopRunning_AndUpdateButton()
        End If
        COM_Close()
        TTS_Stop()
        'EnableAllButtons()
        AllFlashingButtonsToFirstColor()
        If ResetHilight Then HighlightBlock("")
    End Sub

    Friend Sub StopRunning_AndUpdateButton()
        Try
            UpdateOutputBox()
            Running = False
            TestRunningTime()
            UpdateStartStopButtons()
            SetVisibleButtons()
        Catch
        End Try
    End Sub

    Friend Sub TestRunningTime()
        If BlocksCounter = 0 Then Return
        AddTextToOutputBox("Total running time = " + CInt(GetRunningMilliSeconds()).ToString + " mS")
        AddTextToOutputBox("Milliseconds per line = " + (GetRunningMilliSeconds() / BlocksCounter).ToString("0.0", GCI) + vbCrLf)
        UpdateOutputBox()
    End Sub


    ' ===================================================================================================
    '  HOST OBJECT - TWO WAY FUNCTION CALLS -
    '  Used by the Theremino Blocks: Input / Slot / SlotText
    ' ===================================================================================================
    <ComVisible(True)>
    Public Class DomManipulationDotnetInterop
        ' ---------------------------------------------------------- GET VALUE
        Public Function GetValue(ByVal str As String) As String
            AddTextToOutputBox(str)
            Return DecodeCommands_INPUT(str)
        End Function
        ' ---------------------------------------------------------- SET VALUE
        Public Function SetValue(ByVal str As String) As String
            AddTextToOutputBox(str)
            DecodeCommands_OUTPUT(str)
            Return ""
        End Function
        ' ---------------------------------------------------------- Waiting Synthesizer
        Public Function WaitingSynthesizer() As String
            If Running And Not TTS_SynthesizerReady() Then
                Return "true"
            Else
                Return "false"
            End If
        End Function
    End Class

End Class
