﻿
Imports System.Threading.Tasks
Imports Newtonsoft.Json

Module Module_Blockly

    Friend LoopMillisec As Int32
    Friend SleepEveryNloops As Int32

    ' ===================================================================================================
    '  JAVASCRIPT FUNCTION CALLS
    ' ===================================================================================================
    Friend ReadOnly BlocklyFilesPath As String = Application.StartupPath + "\Blockly\Files\"
    Private ToolboxXML As String
    Friend WorkspaceXML As String

    Friend Async Sub SetBlocklyToolboxAndWorkspace()
        Await ExecuteScriptFunctionAsync("initToolboxAndWorkspace",
                                         New String() {ToolboxXML, WorkspaceXML})
    End Sub

    Friend Sub ReadBloclyToolboxFromFile()
        If IO.File.Exists(BlocklyFilesPath + "XML\blockyToolbox.xml") Then
            ToolboxXML = System.IO.File.ReadAllText(BlocklyFilesPath + "XML\blockyToolbox.xml")
        End If
    End Sub

    Public Sub ExecuteScriptFunction(ByVal FunctionName As String)
        Form1.WV2.ExecuteScriptAsync(FunctionName)
    End Sub

    Public Async Sub ExecuteScriptFunctionAsync(ByVal FunctionName As String)
        Await Form1.WV2.ExecuteScriptAsync(FunctionName)
    End Sub

    Friend Async Sub HighlightBlock(id As String)
        Dim s(0) As String
        s(0) = id
        Await ExecuteScriptFunctionAsync("highlightBlock", s)
    End Sub

    Public Async Function ExecuteScriptFunctionAsync(ByVal FunctionName As String,
                                                     ByVal Parameters() As Object) _
                                                     As Threading.Tasks.Task(Of String)
        Dim script As String = FunctionName + "("
        Dim i As Integer = 0
        Do While i < Parameters.Length
            script += JsonConvert.SerializeObject(Parameters(i))
            If i < (Parameters.Length - 1) Then
                script += ", "
            End If
            i += 1
        Loop
        script += ");"

        Dim result As Object = Await Form1.WV2.ExecuteScriptAsync(script)
        result = CType(JsonConvert.DeserializeObject(result.ToString), String)
        Return CStr(result)
    End Function

    ' ===================================================================================================
    '  GET BLOCKLY CODE
    ' ===================================================================================================
    '  Wait until async code is ready 
    ' ===================================================================================================
    Friend Function GetBlocklyCode() As String
        For i As Int32 = 1 To 100 ' <-- timeout 
            GetBlocklyCode2()
            Application.DoEvents()
            Threading.Thread.Sleep(10)
            If BlocklyCode IsNot Nothing AndAlso BlocklyCode <> "" Then
                Return BlocklyCode
            End If
        Next
        Return ""
    End Function

    Private BlocklyCode As String = ""
    Friend Async Sub GetBlocklyCode2()
        BlocklyCode = ""
        Dim result As Object = Await Form1.WV2.ExecuteScriptAsync("getBlocklyCode()")
        BlocklyCode = CType(JsonConvert.DeserializeObject(result.ToString), String)
    End Sub

    Friend Sub ShowBlocklyCode()
        Dim s As String = "Print " + vbCrLf
        s += "=================================================== " + vbCrLf
        s += "  BLOCKLY CODE " + vbCrLf
        s += "=================================================== " + vbCrLf
        s += GetBlocklyCode()
        s += "=================================================== " + vbCrLf
        AddTextToOutputBox(s)
    End Sub

    ' ===================================================================================================
    '  GET BLOCKLY WORKSPACE
    ' ===================================================================================================
    '  With a workaround to get the updated workspace (otherwise last editing are loosed) 
    ' ===================================================================================================
    Friend Sub GetBlocklyWorkspace()
        ' ------------------------------------------ Stop the execution loop
        ExecutionLoopEnabled = False
        ' ------------------------------------------ Wait the workspace
        For i As Int32 = 1 To 100 ' <-- timeout
            GetBlocklyWorkspace2()
            Application.DoEvents()
            Threading.Thread.Sleep(10)
            If WorkspaceXML IsNot Nothing AndAlso
                WorkspaceXML.StartsWith("<xml xmlns=") AndAlso
                WorkspaceXML.EndsWith("</xml>") Then
                'Form1.Text = i.ToString
                'Console.Beep(1500, 25)
                Click2()
                Return
            End If
            Click1c()
        Next
    End Sub
    Private Async Sub GetBlocklyWorkspace2()
        WorkspaceXML = ""
        Dim s As String
        s = Await Form1.WV2.ExecuteScriptAsync("getWorkspaceAsText()")
        s = CType(JsonConvert.DeserializeObject(s), String)
        WorkspaceXML = s
    End Sub

    Friend Sub ShowBlocklyWorkspace()
        GetBlocklyWorkspace()
        Dim s As String = "Print " + vbCrLf
        s += "=================================================== " + vbCrLf
        s += "  BLOCKLY WORKSPACE " + vbCrLf
        s += "=================================================== " + vbCrLf
        s += WorkspaceXML + vbCrLf
        s += "=================================================== " + vbCrLf
        AddTextToOutputBox(s)
    End Sub

    ' ===================================================================================================
    '  GET BLOCKLY TOOLBOX
    ' ===================================================================================================
    'Friend Async Sub GetBlocklyToolbox()
    '    Dim s As String
    '    s = Await Form1.WV2.ExecuteScriptAsync("getToolboxAsText()")
    '    s = CType(JsonConvert.DeserializeObject(s), String)
    '    MessageBox.Show(s + vbCrLf + vbCrLf + s.Length.ToString)
    'End Sub

    ' ===================================================================================================
    '  INITIALIZE BUTTONS USING THE LAST WORKSPACE
    ' ===================================================================================================
    Friend Sub InitializeButtonUsingLastWorkspace()
        ' --------------------------------------------------------
        ' Initialize the interpreter to get the last blockly code
        ' Get the last code with changed buttons
        ' Clear the interpreter stack before the next run
        ' --------------------------------------------------------
        ExecuteScriptFunction("initInterpreter()")
        Form1.InitButtons(GetBlocklyCode())
        ExecuteScriptFunction("clearInterpreterThreads()")
    End Sub

    ' ===================================================================================================
    '  EXECUTE FUNCTIONS BY BUTTONS
    ' ===================================================================================================
    Friend Sub ExecuteFunctionByIdentifier(Identifier As String)
        ' TODO -- Normalize OK ?
        ' -------------------------------------------------------
        Identifier = Identifier.Replace(" ", "_")
        Identifier = Identifier.Replace(vbCrLf, "__")
        Identifier = Identifier.Replace("__", "_")
        ' -------------------------------------------------------
        AddTextToOutputBox("Starting function " + Identifier)
        ' -------------------------------------------------------
        If MainCodePresent() Then
            If Not Running Then
                StartRunningToExecuteFunction()
            End If
            ExecuteScriptFunction("runFunctionBlock('" + Identifier + "')")
        Else
            ExecuteScriptFunction("runFunctionBlock('" + Identifier + "')")
            StartRunningToExecuteFunction()
        End If
    End Sub

    Friend Function MainCodePresent() As Boolean
        If BlocklyCode IsNot Nothing AndAlso
           BlocklyCode.Contains(vbLf + "highlightBlock") Then
            Return True
        Else
            Return False
        End If
    End Function

    Private Sub StartRunningToExecuteFunction()
        If Not Running Then
            Running = True
            Form1.UpdateStartStopButtons()
            StartRunningTime()
            ExecuteScriptFunction("initInterpreter()")
            ExecutionLoop()
        End If
    End Sub


    ' ===================================================================================================
    '  RUN SINGLE STEP
    ' ===================================================================================================
    Friend Async Sub RunSingleStep()
        StartRunningTime()
        Dim retval As String
        retval = Await ExecuteScriptFunctionAsync("runInterpreterStepManual", New String() {""})
        If retval = "PROGRAM_END" Then
            ExecuteScriptFunction("initInterpreter()")
            Await ExecuteScriptFunctionAsync("runInterpretersStepManual", New String() {""})
        End If
        ' ------------------------------------------ Test for PROGRAM_CHANGED and PROGRAM_END
        If retval <> "" Then
            AddTextToOutputBox(retval)
            DecodeCommands_OUTPUT(retval)
        End If
        ' ------------------------------------------ 
        BlocksCounter += 1
        Form1.ShowVariableList()
    End Sub

    ' ===================================================================================================
    '  EXECUTION LOOP
    ' ===================================================================================================
    Friend ExecutionLoopEnabled As Boolean
    Friend BlocksCounter As Int32

    Friend Async Sub ExecutionLoop()
        ' --------------------------------------------------- 
        ExecutionLoopEnabled = True
        BlocksCounter = 0
        StartRunningTime()
        Do
            Dim retval As String
            retval = Await ExecuteScriptFunctionAsync("runInterpreterStep", New String() {""})
            '
            ' ------------------------------------------ Test for PROGRAM_CHANGED and PROGRAM_END
            If retval <> "" Then
                AddTextToOutputBox(retval)
                DecodeCommands_OUTPUT(retval)
            End If
            ' ------------------------------------------ RUNNING SPEED
            If LoopMillisec = 0 Then
                If BlocksCounter Mod SleepEveryNloops = 0 Then
                    Threading.Thread.Sleep(1)
                End If
            Else
                If LoopMillisec < 100 Then
                    Threading.Thread.Sleep(LoopMillisec)
                Else
                    Await Task.Delay(LoopMillisec)
                End If
            End If
            '
            BlocksCounter += 1
            '
        Loop Until Not ExecutionLoopEnabled
        '
        Form1.ShowVariableList()
        Form1.StopRunning_AndUpdateButton()
    End Sub

End Module
