﻿
Module OpenCloseApps

    ' ==============================================================================================================
    '   OPEN ALL THEREMINC APPS (in the APP Folder and Subfolders)
    ' ==============================================================================================================
    Friend Sub OpenAllThereminicApps()
        ' ------------------------------------------------------------------
        FindThereminicApps()
        ' ------------------------------------------------------------------
        For Each s As String In OpenedProcessesList.Keys
            If ProcessRunning(s) Then Continue For
            Dim p As Process = New Process()
            p.StartInfo.WorkingDirectory = Application.StartupPath
            p.StartInfo.FileName = s
            p.StartInfo.UseShellExecute = False
            p.Start()
        Next
    End Sub

    ' ===========================================================
    '   Find Applications in the APP Folder and Subfolders
    ' ===========================================================
    Private OpenedProcessesList As Generic.Dictionary(Of String, Int32) = New Generic.Dictionary(Of String, Int32)

    Friend Sub ClearTheOpenedProcessesList()
        OpenedProcessesList.Clear()
    End Sub

    Friend Sub AddProcessToOpenedProcessesList(ByVal filename As String)
        If Not OpenedProcessesList.ContainsKey(filename) Then
            OpenedProcessesList.Add(filename, 0)
        End If
    End Sub

    Private Sub FindThereminicApps()
        Dim wildcards() As String = {"Theremino_*.exe"}
        Dim s As String = Application.StartupPath & "\Apps\"
        If Not My.Computer.FileSystem.DirectoryExists(s) Then Return
        Dim fc As Collections.ObjectModel.ReadOnlyCollection(Of String)
        fc = My.Computer.FileSystem.GetFiles(s, FileIO.SearchOption.SearchAllSubDirectories, wildcards)
        For i As Int32 = 0 To fc.Count - 1
            s = fc(i).ToLower
            If Not s.Contains("vshost") And Not s.Contains("\sources\") And Not s.Contains("\obj\") Then
                If Not s.ToLower = System.Windows.Forms.Application.ExecutablePath.ToLower Then
                    ' ------------------------------------------------------------------ Add to OpenedProcesses dictionary (if the key is not present)
                    If Not OpenedProcessesList.ContainsKey(s) Then
                        OpenedProcessesList.Add(s, 0)
                    End If
                End If
            End If
        Next
    End Sub

    ' ======================================================================
    '   Find File Path by Name in the specified Folder and Subfolders
    ' ======================================================================
    Friend Function FindFilePathByName(ByVal appname As String,
                                       ByVal StartingFolder As String) As String
        If appname = "" Then Return ""
        appname = IO.Path.GetFileName(appname)
        If appname = "" Then Return ""
        Dim wildcards() As String = {appname}
        If Not StartingFolder.EndsWith("\") Then StartingFolder += "\"
        Dim fc As Collections.ObjectModel.ReadOnlyCollection(Of String)
        fc = My.Computer.FileSystem.GetFiles(StartingFolder,
                                             FileIO.SearchOption.SearchAllSubDirectories,
                                             wildcards)
        '
        For i As Int32 = 0 To fc.Count - 1
            Dim s As String = fc(i).ToLower
            If IO.Path.GetFileName(s).StartsWith("theremino") Then
                If Not s.Contains("vshost") And Not s.Contains("\sources\") And Not s.Contains("\obj\") Then
                    Return fc(i)
                End If
            Else
                Return fc(i)
            End If
        Next
        '
        Return appname
    End Function


    ' =============================================================================
    '   NORMALIZE FILENAME
    ' =============================================================================
    Friend Function NormalizeFilename(ByVal filename As String) As String
        ' ------------------------------------------------------------------ Prepare path
        If Not IO.Path.IsPathRooted(filename) Then
            filename = Application.StartupPath + "\" + filename
        End If
        ' ------------------------------------------------------------------ Correct path for /../..
        filename = IO.Path.GetFullPath(filename)
        ' ------------------------------------------------------------------ Normalize filename
        filename = filename.Replace("/", "\")
        Return (filename)
    End Function


    ' =============================================================================
    '   Process Running  (testing the complete path of the running app)
    ' =============================================================================
    Friend Function ProcessRunning(ByVal FileName As String) As Boolean
        If FileName = "" Then Return False
        FileName = NormalizeFilename(FileName)
        ' -------------------------------------------------------------- FIND FILE IF NOT EXISTS
        If Not IO.File.Exists(FileName) Then
            FileName = FindFilePathByName(FileName, Application.StartupPath + "\Apps\")
        End If
        ' ------------------------------------------------------------------ Prepare ProcessName to compare
        Dim ProcessName As String = IO.Path.GetFileNameWithoutExtension(FileName)
        ' ------------------------------------------------------------------ Compare with all processes
        Dim p() As System.Diagnostics.Process
        p = System.Diagnostics.Process.GetProcesses()
        For i As Int32 = 0 To p.Length - 1
            Try
                ' ---------------------------------------------------------- test ProcessName for fast execution
                If Not p(i).ProcessName.ToLower = ProcessName.ToLower Then
                    Continue For
                End If
                ' ---------------------------------------------------------- test f
                If p(i).MainModule.FileName.ToLower = FileName.ToLower Then
                    Return True
                End If
            Catch
            End Try
        Next
        Return False
    End Function

    ' ==============================================================================================================
    '   CLOSE ALL OPENED PROCESSES
    ' ==============================================================================================================
    Friend Sub CloseAllOpenedProcesses()
        Dim ProcessName As String
        Dim p() As System.Diagnostics.Process
        p = System.Diagnostics.Process.GetProcesses()
        For i As Int32 = 0 To p.Length - 1
            If p(i).Id = System.Diagnostics.Process.GetCurrentProcess.Id Then Continue For
            '
            For Each OpenedProc As System.Collections.Generic.KeyValuePair(Of String, Int32) In OpenedProcessesList
                ' ---------------------------------------------------------- test ProcessName for fast execution
                ProcessName = IO.Path.GetFileNameWithoutExtension(OpenedProc.Key)
                If Not p(i).ProcessName.ToLower = ProcessName.ToLower Then
                    Continue For
                End If
                Try
                    ' ---------------------------------------------------------- test f
                    If p(i).MainModule.FileName.ToLower = OpenedProc.Key.ToLower Then
                        '
                        'Debug.Print(p(i).ProcessName)
                        '
                        p(i).CloseMainWindow()
                        p(i).Close()
                        Exit For
                    End If
                Catch
                    ' --------------------- some process as Notepad generates errors when reading the MainModule
                    p(i).CloseMainWindow()
                    p(i).Close()
                    Exit For
                End Try
            Next
        Next
        '
        p = System.Diagnostics.Process.GetProcesses()
        For i As Int32 = 0 To p.Length - 1
            '
            If p(i).Id = System.Diagnostics.Process.GetCurrentProcess.Id Then Continue For
            '
            For Each OpenedProc As System.Collections.Generic.KeyValuePair(Of String, Int32) In OpenedProcessesList
                ' ---------------------------------------------------------- test ProcessName for fast execution
                ProcessName = IO.Path.GetFileNameWithoutExtension(OpenedProc.Key)
                If Not p(i).ProcessName.ToLower = ProcessName.ToLower Then
                    Continue For
                End If
                ' ---------------------------------------------------------- test f
                Try
                    If Not p(i).HasExited Then
                        If p(i).MainModule.FileName.ToLower = OpenedProc.Key.ToLower Then
                            '
                            'Debug.Print(p(i).ProcessName)
                            '
                            If Not p(i).WaitForExit(1000) Then
                                Try
                                    p(i).Kill()
                                Catch ex As Exception
                                End Try
                            End If
                            Exit For
                        End If
                    End If
                Catch
                End Try
            Next
        Next
    End Sub


    ' =============================================================================
    '   START PROCESS (IF NOT RUNNING) AND WAIT
    ' =============================================================================
    Friend Sub StartProcess_IfNotRunning_AndWait(ByVal filename As String, ByVal Wait As Boolean)
        ' ------------------------------------------------------------------ Prepare path
        If filename = "" Then Return
        filename = NormalizeFilename(filename)
        If Not IO.File.Exists(filename) Then Return
        ' ------------------------------------------------------------------ Add to OpenedProcesses dictionary
        AddProcessToOpenedProcessesList(filename)
        ' ------------------------------------------------------------------ Test if already running
        If ProcessRunning(filename) Then Return
        ' ------------------------------------------------------------------ Init process
        Dim proc As Process
        Dim psi As New ProcessStartInfo
        psi.WorkingDirectory = IO.Path.GetDirectoryName(filename)
        psi.FileName = filename
        ' ------------------------------------------------------------------ Protect for process errors
        Try
            ' -------------------------------------------------------------- Start process
            proc = Process.Start(psi)
            ' -------------------------------------------------------------- Wait the process really started 
            If Wait Then
                For i As Int32 = 1 To 200  ' 200x50mS = 10 to 15 seconds
                    Threading.Thread.Sleep(50)
                    proc.Refresh()
                    Try
                        If proc.MainWindowTitle.Length > 1 Then
                            Return
                        End If
                    Catch
                    End Try
                Next
            End If
        Catch
        End Try
    End Sub


    ' =============================================================================
    '   START PROCESS (ALSO IF RUNNING) AND WAIT
    ' =============================================================================
    Friend Sub StartProcess_AlsoIfRunning_AndWait(ByVal filename As String, ByVal Wait As Boolean)
        ' ------------------------------------------------------------------ Prepare path
        If filename = "" Then Return
        filename = NormalizeFilename(filename)
        If Not IO.File.Exists(filename) Then Return
        ' ------------------------------------------------------------------ Init process
        Dim proc As Process
        Dim psi As New ProcessStartInfo
        psi.WorkingDirectory = IO.Path.GetDirectoryName(filename)
        psi.FileName = filename
        ' ------------------------------------------------------------------ Protect for process errors
        Try
            ' -------------------------------------------------------------- Start process
            proc = Process.Start(psi)
            ' -------------------------------------------------------------- Wait the process really started 
            If Wait Then
                For i As Int32 = 1 To 200  ' 200x50mS = 10 to 15 seconds
                    Threading.Thread.Sleep(50)
                    proc.Refresh()
                    Try
                        If proc.MainWindowTitle.Length > 1 Then
                            Return
                        End If
                    Catch
                    End Try
                Next
            End If
        Catch
        End Try
    End Sub


    ' ========================================================
    '   PROCESS START AND WAIT PROCESS CLOSED
    ' ========================================================
    Friend Sub ProcessStartAndWait(ByVal fname As String)
        If Not My.Computer.FileSystem.FileExists(fname) Then Return
        Dim p As Process = Process.Start(fname)
        Do
            System.Threading.Thread.Sleep(100)
        Loop Until p.HasExited
    End Sub

    ' ========================================================
    '   PROCESS START  AND WAIT PROCESS CLOSED (NOTEPAD)
    ' ========================================================
    Friend Sub NotepadProcess_StartAndWait(ByVal fname As String)
        If Not My.Computer.FileSystem.FileExists(fname) Then Return
        Dim p As Process = Process.Start("notepad.exe", fname)
        Do
            System.Threading.Thread.Sleep(100)
        Loop Until p.HasExited
    End Sub


    ' =============================================================================
    '   Close App By Filename  (using ProcessName for fast execution)
    ' -----------------------------------------------------------------------------
    '  Valid paths are:
    '   - A complete path, for example C:\xxx\yyyy.exe
    '   - A partial path,  for example Files\yyyy.exe
    '   - A process name,  for example yyyy.exe
    '   - A process name,  for example yyyy
    ' =============================================================================
    Friend Sub CloseAppByFilename_LowLevel(ByVal FileName As String)
        If FileName = "" Then Return
        ' ------------------------------------------------------------------ transform statements and formula to a single string
        'f = EvalString_AutoQuoting(f)
        ' ------------------------------------------------------------------ if the path is rooted the file must exist
        If IO.Path.IsPathRooted(FileName) Then
            If Not IO.File.Exists(FileName) Then Return
        Else
            ' -------------------------------------------------------------- else try to normalize it to automation folders
            Dim f As String = NormalizeFilename(FileName)
            If IO.File.Exists(f) Then
                FileName = f
            End If
        End If
        ' ------------------------------------------------------------------ if the file exists then use the process-name
        Dim p() As System.Diagnostics.Process
        p = System.Diagnostics.Process.GetProcesses()
        Dim ProcessName As String = IO.Path.GetFileNameWithoutExtension(FileName)
        For i As Int32 = 0 To p.Length - 1
            Try
                If p(i).Id = System.Diagnostics.Process.GetCurrentProcess.Id Then Continue For

                ' ----------------------------------------------------------------------------------
                ' Sometimes not working because the ProcessName is different from filename  
                ' ----------------------------------------------------------------------------------
                'If p(i).ProcessName.ToLower.StartsWith("calc") Then
                '    Beep()
                'End If

                If Not p(i).ProcessName = ProcessName Then Continue For
                '
                'Debug.Print(p(i).ProcessName)
                '
                p(i).CloseMainWindow()
                p(i).Close()
            Catch
            End Try
        Next
        '
        p = System.Diagnostics.Process.GetProcesses()
        For i As Int32 = 0 To p.Length - 1
            Try
                If p(i).Id = System.Diagnostics.Process.GetCurrentProcess.Id Then Continue For
                If Not p(i).ProcessName = ProcessName Then Continue For
                '
                'Debug.Print(p(i).ProcessName)
                '
                If Not p(i).WaitForExit(1000) Then
                    Try
                        p(i).Kill()
                    Catch ex As Exception
                    End Try
                End If
            Catch
            End Try
        Next
    End Sub


    ' =============================================================================
    '   Open App By Filename  (using ProcessName for fast execution)
    ' =============================================================================
    Friend Sub OpenAppByFilename(ByVal filename As String)
        Dim f As String = ""
        If IO.File.Exists(filename) Then
            StartProcess_IfNotRunning_AndWait(filename, False)
            Form1.Activate()
            Return
        End If
        ' ------------------------------------------------------- FIND FILE IN APPS and MEDIA
        If Not FileExists_RemovingCmdLine(filename) Then
            f = FindFilePathByName(filename, Application.StartupPath + "\Apps\")
        End If
        If Not FileExists_RemovingCmdLine(f) Then
            f = FindFilePathByName(filename, Application.StartupPath + "\Media\")
        End If
        ' ------------------------------------------------------- Open 
        If IO.File.Exists(f) Then
            StartProcess_IfNotRunning_AndWait(f, False)
            Return
        End If
        ' ------------------------------------------------------- test also if it is a folder
        If My.Computer.FileSystem.DirectoryExists(f) Then
            Process.Start(f)
            Form1.Activate()
            Return
        End If
        f = Application.StartupPath + "\" + filename.Replace("""", "") '+ "\"
        If My.Computer.FileSystem.DirectoryExists(f) Then
            Process.Start(f)
            Form1.Activate()
            Return
        End If
        ' ------------------------------------------------------- otherwise open it with the default Windows editor (maybe NotePad)
        If Not My.Computer.FileSystem.FileExists(f) Then
            f = PlatformAdjustedFileName(Application.StartupPath + "\Files\" + RemoveFilesPrefix(filename))
        End If
        If Not My.Computer.FileSystem.FileExists(f) Then
            f = IO.Path.GetDirectoryName(ProgramFile_PathAndName) + "\" + filename
        End If
        If My.Computer.FileSystem.FileExists(f) Then
            Process.Start(f)
            Form1.Activate()
            Return
        End If
        ' ------------------------------------------------------- test also if it is a folder
        f = Application.StartupPath + "\" + filename.Replace("""", "") '+ "\"
        If My.Computer.FileSystem.DirectoryExists(f) Then
            Process.Start(f)
            Form1.Activate()
        End If
    End Sub

    Private Function RemoveProgramsPrefix(ByRef str As String) As String
        str.Replace("/", "\")
        If str.ToLower.StartsWith("programs\") Then
            str = Strings.Replace(str, "programs\", "", , , CompareMethod.Text)
        End If
        Return str
    End Function

    Private Function RemoveFilesPrefix(ByRef str As String) As String
        str.Replace("/", "\")
        If str.ToLower.StartsWith("files\") Then
            str = Strings.Replace(str, "files\", "", , , CompareMethod.Text)
        End If
        Return str
    End Function

    Private Function RemoveMediaPrefix(ByRef str As String) As String
        str.Replace("/", "\")
        If str.ToLower.StartsWith("media\") Then
            str = Strings.Replace(str, "media\", "", , , CompareMethod.Text)
        End If
        Return str
    End Function


    ' =============================================================================
    '   Close App By Filename  (using ProcessName for fast execution)
    ' =============================================================================
    Friend Sub CloseAppByFilename(ByVal FileName As String)
        ' -------------------------------------------------------------- FIND FILE IF NOT EXISTS
        If Not FileExists_RemovingCmdLine(FileName) Then
            FileName = FindFilePathByName(FileName, Application.StartupPath + "\Apps\")
        End If
        CloseAppByFilename_LowLevel(FileName)
        Form1.Activate()
    End Sub


    Private Function FileExists_RemovingCmdLine(ByVal FilePathAndCmdLine As String) As Boolean

        If FilePathAndCmdLine = "" Then Return False

        Dim sa() As String = FilePathAndCmdLine.Split(" "c)
        sa(0) = sa(0).Replace("""", "")
        ' -------------------------------------------------------------- Get full path
        FilePathAndCmdLine = IO.Path.GetFullPath(sa(0))
        ' -------------------------------------------------------------- Test if the path is an existent directory
        If IO.Directory.Exists(sa(0)) Then Return True
        ' -------------------------------------------------------------- Test if the path is an existent file
        If IO.File.Exists(sa(0)) Then Return True
        ' -------------------------------------------------------------- remove the ".exe" and test if the file exists
        Dim i As Int32 = FilePathAndCmdLine.IndexOf(".exe")
        If i > 0 Then
            sa(0) = Strings.Left(sa(0), i + 4)
        End If
        Return My.Computer.FileSystem.FileExists(sa(0))
    End Function



End Module
