
// ====================================================================================== 
//  CUSTOM BLOCKS
//  https://developers.google.com/blockly/guides/configure/web/custom-blocks#javascript
//  https://developers.google.com/blockly/guides/create-custom-blocks/generating-code
// ======================================================================================

function initGlobal(interpreter, globalObject)
{
    // Ensure function names does not conflict with variable names.

    Blockly.JavaScript.addReservedWords('exit'); // used to signal the end of a script.

    Blockly.JavaScript.addReservedWords('highlightblock');
    Blockly.JavaScript.addReservedWords('waitForSeconds');
    
    Blockly.JavaScript.addReservedWords('print');
    Blockly.JavaScript.addReservedWords('beep');
    Blockly.JavaScript.addReservedWords('read_slot');
    Blockly.JavaScript.addReservedWords('read_slottext');
    Blockly.JavaScript.addReservedWords('write_slot');
    Blockly.JavaScript.addReservedWords('write_slottext');
    
    Blockly.JavaScript.addReservedWords('openapp');
    Blockly.JavaScript.addReservedWords('closeapp');
    Blockly.JavaScript.addReservedWords('openapps');
    Blockly.JavaScript.addReservedWords('closeapps');
    Blockly.JavaScript.addReservedWords('closeall');
    
    Blockly.JavaScript.addReservedWords('tts_volume');
    Blockly.JavaScript.addReservedWords('tts_speed');
    Blockly.JavaScript.addReservedWords('tts_voice');
    Blockly.JavaScript.addReservedWords('tts_speak');
    Blockly.JavaScript.addReservedWords('tts_voices');
    Blockly.JavaScript.addReservedWords('tts_stop');
    Blockly.JavaScript.addReservedWords('tts_pause');
    Blockly.JavaScript.addReservedWords('tts_resume');
    Blockly.JavaScript.addReservedWords('tts_wait');

    Blockly.JavaScript.addReservedWords('com_portnames');
    Blockly.JavaScript.addReservedWords('com_open');
    Blockly.JavaScript.addReservedWords('com_close');
    Blockly.JavaScript.addReservedWords('com_status');
    Blockly.JavaScript.addReservedWords('com_sendtext');
    Blockly.JavaScript.addReservedWords('com_sendtextdouble');
    Blockly.JavaScript.addReservedWords('com_received');
    Blockly.JavaScript.addReservedWords('com_receiveuntil');
    Blockly.JavaScript.addReservedWords('com_receivenchars');
    Blockly.JavaScript.addReservedWords('com_discardinbuffer');
    Blockly.JavaScript.addReservedWords('com_discardoutbuffer');
    Blockly.JavaScript.addReservedWords('com_enabledtr');
    Blockly.JavaScript.addReservedWords('com_enablerts');
    Blockly.JavaScript.addReservedWords('com_disabledtr');
    Blockly.JavaScript.addReservedWords('com_disablerts');
    Blockly.JavaScript.addReservedWords('com_getdsr');
    Blockly.JavaScript.addReservedWords('com_getcts');

    Blockly.JavaScript.addReservedWords('enable_all_buttons');
    Blockly.JavaScript.addReservedWords('disable_all_buttons');
    Blockly.JavaScript.addReservedWords('enable_button');
    Blockly.JavaScript.addReservedWords('disable_button');
    Blockly.JavaScript.addReservedWords('button_colour');
    Blockly.JavaScript.addReservedWords('button_colour_flashing');
    Blockly.JavaScript.addReservedWords('button_slot');
    Blockly.JavaScript.addReservedWords('button_text');

    Blockly.JavaScript.addReservedWords('open_textbox');
    Blockly.JavaScript.addReservedWords('close_textbox');
    Blockly.JavaScript.addReservedWords('clear_textbox');


    // =================================================
    //   HIGHLIGHT BLOCKS
    // =================================================
    // -------------------------------------------------------------- Add API for highlighting blocks
    var wrapper = function(id) 
    {
        id = id ? id.toString() : '';
        return interpreter.createPrimitive(highlightBlock(id));
    };
    interpreter.setProperty(globalObject, 'highlightBlock',
                           interpreter.createNativeFunction(wrapper));

    // =================================================
    //   WAIT
    // =================================================
    // -------------------------------------------------------------- Add API for the "waitForSeconds" block
    var wrapper = interpreter.createAsyncFunction
    (
        function(timeInSeconds, callback) 
        {     
            // ------------------------------------------------------ Delay the call to the callback.
            if (RunningManualStep == 'true')
            {
                ExecutingWaitBlock = false;
                callback();
            }
            else
            {
                ExecutingWaitBlock = true;
                setTimeout(callback, timeInSeconds * 1000);
            }
        }
    );
    interpreter.setProperty(globalObject, 'waitForSeconds', wrapper);


    // =================================================
    //   IN OUT
    // =================================================
    // -------------------------------------------------------------- Add API for the "print" blocks
    var wrapper = function(text)
    {
        text = text ? text.toString() : '';
        var dummy = chrome.webview.hostObjects.sync.dotnet.SetValue('Print ' + text);
    };
    interpreter.setProperty(globalObject, 'print',
                            interpreter.createNativeFunction(wrapper));

    // -------------------------------------------------------------- Add API for the "beep" blocks
    var wrapper = function(t1, t2)
    {
        t1 = t1 ? t1.toString() : '0';
        t2 = t2 ? t2.toString() : '0';
        var dummy = chrome.webview.hostObjects.sync.dotnet.SetValue('Beep ' + t1 + ' ' + t2);
    };
    interpreter.setProperty(globalObject, 'beep',
                            interpreter.createNativeFunction(wrapper));

    // =================================================
    //   SLOTS
    // =================================================
    // -------------------------------------------------------------- Add API for the "read_slot" blocks
    var wrapper = function(text)
    {
        text = text ? text.toString() : '0';
        var msg = chrome.webview.hostObjects.sync.dotnet.GetValue('read_slot ' + text);
        return msg;
    };
    interpreter.setProperty(globalObject, 'read_slot',
                            interpreter.createNativeFunction(wrapper));

    // -------------------------------------------------------------- Add API for the "read_slottext" blocks
    var wrapper = function(text)
    {
        text = text ? text.toString() : '';
        var msg = chrome.webview.hostObjects.sync.dotnet.GetValue('read_slottext ' + text);
        return msg;
    };
    interpreter.setProperty(globalObject, 'read_slottext',
                            interpreter.createNativeFunction(wrapper));

    // -------------------------------------------------------------- Add API for the "write_slot" blocks
    var wrapper = function(t1, t2)
    {
        t1 = t1 ? t1.toString() : '0';
        t2 = t2 ? t2.toString() : '0';
        var dummy = chrome.webview.hostObjects.sync.dotnet.SetValue('write_slot ' + t1 + ' ' + t2);
    };
    interpreter.setProperty(globalObject, 'write_slot',
                            interpreter.createNativeFunction(wrapper));

    // -------------------------------------------------------------- Add API for the "write_slottext" blocks
    var wrapper = function(t1, t2)
    {
        t1 = t1 ? t1.toString() : '0';
        t2 = t2 ? t2.toString() : '';
        var dummy = chrome.webview.hostObjects.sync.dotnet.SetValue('write_slotText ' + t1 + ' ' + t2);
    };
    interpreter.setProperty(globalObject, 'write_slottext',
                            interpreter.createNativeFunction(wrapper));    

    // =================================================
    //    APPS
    // =================================================
    // -------------------------------------------------------------- Add API for the "openapp" blocks
    var wrapper = function(text)
    {
        text = text ? text.toString() : '';
        var dummy = chrome.webview.hostObjects.sync.dotnet.SetValue('open_app ' + text);
    };
    interpreter.setProperty(globalObject, 'open_app',
                            interpreter.createNativeFunction(wrapper));

    // -------------------------------------------------------------- Add API for the "closeapp" blocks
    var wrapper = function(text)
    {
        text = text ? text.toString() : '';
        var dummy = chrome.webview.hostObjects.sync.dotnet.SetValue('close_app ' + text);
    };
    interpreter.setProperty(globalObject, 'close_app',
                            interpreter.createNativeFunction(wrapper));

    // -------------------------------------------------------------- Add API for the "openapps" blocks
    var wrapper = function()
    {
        var dummy = chrome.webview.hostObjects.sync.dotnet.SetValue('open_apps');
    };
    interpreter.setProperty(globalObject, 'open_apps',
                            interpreter.createNativeFunction(wrapper));

    // -------------------------------------------------------------- Add API for the "closeapps" blocks
    var wrapper = function()
    {
        var dummy = chrome.webview.hostObjects.sync.dotnet.SetValue('close_apps');
    };
    interpreter.setProperty(globalObject, 'close_apps',
                            interpreter.createNativeFunction(wrapper));

    // -------------------------------------------------------------- Add API for the "closeall" blocks
    var wrapper = function()
    {
        var dummy = chrome.webview.hostObjects.sync.dotnet.SetValue('close_all');
    };
    interpreter.setProperty(globalObject, 'close_all',
                            interpreter.createNativeFunction(wrapper));

    // =================================================
    //    SPEECH
    // =================================================
    // -------------------------------------------------------------- Add API for the "tts_volume" blocks
    var wrapper = function(t1)
    {
        t1 = t1 ? t1.toString() : '0';
        var dummy = chrome.webview.hostObjects.sync.dotnet.SetValue('tts_volume ' + t1);
    };
    interpreter.setProperty(globalObject, 'tts_volume',
                            interpreter.createNativeFunction(wrapper));

    // -------------------------------------------------------------- Add API for the "tts_speed" blocks
    var wrapper = function(t1)
    {
        t1 = t1 ? t1.toString() : '0';
        var dummy = chrome.webview.hostObjects.sync.dotnet.SetValue('tts_speed ' + t1);
    };
    interpreter.setProperty(globalObject, 'tts_speed',
                            interpreter.createNativeFunction(wrapper));

    // -------------------------------------------------------------- Add API for the "tts_voice" blocks
    var wrapper = function(t1)
    {
        t1 = t1 ? t1.toString() : '';
        var dummy = chrome.webview.hostObjects.sync.dotnet.SetValue('tts_voice ' + t1);
    };
    interpreter.setProperty(globalObject, 'tts_voice',
                            interpreter.createNativeFunction(wrapper));

    // -------------------------------------------------------------- Add API for the "tts_speak" blocks
    var wrapper = function(t1)
    {
        t1 = t1 ? t1.toString() : '';
        var dummy = chrome.webview.hostObjects.sync.dotnet.SetValue('tts_speak ' + t1);
    };
    interpreter.setProperty(globalObject, 'tts_speak',
                            interpreter.createNativeFunction(wrapper));

    // -------------------------------------------------------------- Add API for the "tts_voices" blocks
    var wrapper = function()
    {
        var msg = chrome.webview.hostObjects.sync.dotnet.GetValue('tts_voices');
        return msg;
    };
    interpreter.setProperty(globalObject, 'tts_voices',
                            interpreter.createNativeFunction(wrapper));

    // -------------------------------------------------------------- Add API for the "tts_stop" blocks
    var wrapper = function()
    {
        var dummy = chrome.webview.hostObjects.sync.dotnet.SetValue('tts_stop');
    };
    interpreter.setProperty(globalObject, 'tts_stop',
                            interpreter.createNativeFunction(wrapper));
    
    // -------------------------------------------------------------- Add API for the "tts_pause" blocks
    var wrapper = function()
    {
        var dummy = chrome.webview.hostObjects.sync.dotnet.SetValue('tts_pause');
    };
    interpreter.setProperty(globalObject, 'tts_pause',
                            interpreter.createNativeFunction(wrapper));

    // -------------------------------------------------------------- Add API for the "tts_resume" blocks
    var wrapper = function()
    {
        var dummy = chrome.webview.hostObjects.sync.dotnet.SetValue('tts_resume');
    };
    interpreter.setProperty(globalObject, 'tts_resume',
                            interpreter.createNativeFunction(wrapper));
    
    // -------------------------------------------------------------- Add API for the "tts_wait" blocks
    var wrapper = function()
    {
        do
          var waiting = chrome.webview.hostObjects.sync.dotnet.WaitingSynthesizer();
        while (waiting == 'true');
    };
    interpreter.setProperty(globalObject, 'tts_wait',
                            interpreter.createNativeFunction(wrapper));



    // =================================================
    //    COM
    // =================================================
    // -------------------------------------------------------------- Add API for the "com_portnames" blocks
    var wrapper = function()
    {
        var msg = chrome.webview.hostObjects.sync.dotnet.GetValue('com_portnames');
        return msg;
    };
    interpreter.setProperty(globalObject, 'com_portnames',
                            interpreter.createNativeFunction(wrapper));

    // -------------------------------------------------------------- Add API for the "com_open" blocks
    var wrapper = function(t1, t2)
    {
        t1 = t1 ? t1.toString() : '';
        t2 = t2 ? t2.toString() : '9600';
        var dummy = chrome.webview.hostObjects.sync.dotnet.SetValue('com_open ' + t1 + ' ' + t2);
    };
    interpreter.setProperty(globalObject, 'com_open',
                            interpreter.createNativeFunction(wrapper));


    // -------------------------------------------------------------- Add API for the "com_close" blocks
    var wrapper = function()
    {
        var dummy = chrome.webview.hostObjects.sync.dotnet.SetValue('com_close');
    };
    interpreter.setProperty(globalObject, 'com_close',
                            interpreter.createNativeFunction(wrapper));

    // -------------------------------------------------------------- Add API for the "com_status" blocks
    var wrapper = function()
    {
        var msg = chrome.webview.hostObjects.sync.dotnet.GetValue('com_status');
        return msg;
    };
    interpreter.setProperty(globalObject, 'com_status',
                            interpreter.createNativeFunction(wrapper));

    // -------------------------------------------------------------- Add API for the "com_sendtext" blocks
    var wrapper = function(t1)
    {
        t1 = t1 ? t1.toString() : '';
        var dummy = chrome.webview.hostObjects.sync.dotnet.SetValue('com_sendtext ' + t1);
    };
    interpreter.setProperty(globalObject, 'com_sendtext',
                            interpreter.createNativeFunction(wrapper));

    // -------------------------------------------------------------- Add API for the "com_sendtextdouble" blocks
    var wrapper = function(t1, t2)
    {
        t1 = t1 ? t1.toString() : '';
        t2 = t2 ? t2.toString() : '9600';
        var dummy = chrome.webview.hostObjects.sync.dotnet.SetValue('com_sendtextdouble ' + t1 + ' ' + t2);
    };
    interpreter.setProperty(globalObject, 'com_sendtextdouble',
                            interpreter.createNativeFunction(wrapper));

    // -------------------------------------------------------------- Add API for the "com_received" blocks
    var wrapper = function()
    {
        var msg = chrome.webview.hostObjects.sync.dotnet.GetValue('com_received');
        return msg;
    };
    interpreter.setProperty(globalObject, 'com_received',
                            interpreter.createNativeFunction(wrapper));

    // -------------------------------------------------------------- Add API for the "com_receiveuntil" blocks
    var wrapper = function(t1)
    {
        t1 = t1 ? t1.toString() : '';
        var msg = chrome.webview.hostObjects.sync.dotnet.GetValue('com_receiveuntil ' + t1);
        return msg;
    };
    interpreter.setProperty(globalObject, 'com_receiveuntil',
                            interpreter.createNativeFunction(wrapper));

    // -------------------------------------------------------------- Add API for the "com_receivenchars" blocks
    var wrapper = function(t1)
    {
        t1 = t1 ? t1.toString() : '0';
        var msg = chrome.webview.hostObjects.sync.dotnet.GetValue('com_receivenchars ' + t1);
        return msg;
    };
    interpreter.setProperty(globalObject, 'com_receivenchars',
                            interpreter.createNativeFunction(wrapper));


    // -------------------------------------------------------------- Add API for the "com_discardinbuffer" blocks
    var wrapper = function()
    {
        var dummy = chrome.webview.hostObjects.sync.dotnet.SetValue('com_discardinbuffer');
    };
    interpreter.setProperty(globalObject, 'com_discardinbuffer',
                            interpreter.createNativeFunction(wrapper));

    // -------------------------------------------------------------- Add API for the "com_discardoutbuffer" blocks
    var wrapper = function()
    {
        var dummy = chrome.webview.hostObjects.sync.dotnet.SetValue('com_discardoutbuffer');
    };
    interpreter.setProperty(globalObject, 'com_discardoutbuffer',
                            interpreter.createNativeFunction(wrapper));

    // -------------------------------------------------------------- Add API for the "com_enabledtr" blocks
    var wrapper = function()
    {
        var dummy = chrome.webview.hostObjects.sync.dotnet.SetValue('com_enabledtr');
    };
    interpreter.setProperty(globalObject, 'com_enabledtr',
                            interpreter.createNativeFunction(wrapper));

    // -------------------------------------------------------------- Add API for the "com_enablerts" blocks
    var wrapper = function()
    {
        var dummy = chrome.webview.hostObjects.sync.dotnet.SetValue('com_enablerts');
    };
    interpreter.setProperty(globalObject, 'com_enablerts',
                            interpreter.createNativeFunction(wrapper));

    // -------------------------------------------------------------- Add API for the "com_disabledtr" blocks
    var wrapper = function()
    {
        var dummy = chrome.webview.hostObjects.sync.dotnet.SetValue('com_disabledtr');
    };
    interpreter.setProperty(globalObject, 'com_disabledtr',
                            interpreter.createNativeFunction(wrapper));

    // -------------------------------------------------------------- Add API for the "com_disablerts" blocks
    var wrapper = function()
    {
        var dummy = chrome.webview.hostObjects.sync.dotnet.SetValue('com_disablerts');
    };
    interpreter.setProperty(globalObject, 'com_disablerts',
                            interpreter.createNativeFunction(wrapper));

     // -------------------------------------------------------------- Add API for the "com_getdsr" blocks
    var wrapper = function()
    {
        var msg = chrome.webview.hostObjects.sync.dotnet.GetValue('com_getdsr');
        return msg;
    };
    interpreter.setProperty(globalObject, 'com_getdsr',
                            interpreter.createNativeFunction(wrapper));

    // -------------------------------------------------------------- Add API for the "com_getcts" blocks
    var wrapper = function()
    {
        var msg = chrome.webview.hostObjects.sync.dotnet.GetValue('com_getcts');
        return msg;
    };
    interpreter.setProperty(globalObject, 'com_getcts',
                            interpreter.createNativeFunction(wrapper));

    // =================================================
    //    BUTTONS
    // =================================================
    // -------------------------------------------------------------- Add API for the "enable_all_buttons" blocks
    var wrapper = function()
    {
        var msg = chrome.webview.hostObjects.sync.dotnet.SetValue('enable_all_buttons');
        return msg;
    };
    interpreter.setProperty(globalObject, 'enable_all_buttons',
                            interpreter.createNativeFunction(wrapper));

    // -------------------------------------------------------------- Add API for the "disable_all_buttons" blocks
    var wrapper = function()
    {
        var msg = chrome.webview.hostObjects.sync.dotnet.SetValue('disable_all_buttons');
        return msg;
    };
    interpreter.setProperty(globalObject, 'disable_all_buttons',
                            interpreter.createNativeFunction(wrapper));

    // -------------------------------------------------------------- Add API for the "enable_button" blocks
    var wrapper = function(t1)
    {
        t1 = t1 ? t1.toString() : '';
        var dummy = chrome.webview.hostObjects.sync.dotnet.SetValue('enable_button ' + t1);
    };
    interpreter.setProperty(globalObject, 'enable_button',
                            interpreter.createNativeFunction(wrapper));

    // -------------------------------------------------------------- Add API for the "disable_button" blocks
    var wrapper = function(t1)
    {
        t1 = t1 ? t1.toString() : '';
        var dummy = chrome.webview.hostObjects.sync.dotnet.SetValue('disable_button ' + t1);
    };
    interpreter.setProperty(globalObject, 'disable_button',
                            interpreter.createNativeFunction(wrapper));

    // -------------------------------------------------------------- Add API for the "button_colour" blocks
    var wrapper = function(t1, t2)
    {
        t1 = t1 ? t1.toString() : '';
        t2 = t2 ? t2.toString() : '';
        var dummy = chrome.webview.hostObjects.sync.dotnet.SetValue('button_colour ' + t1 + ' ' + t2);
    };
    interpreter.setProperty(globalObject, 'button_colour',
                            interpreter.createNativeFunction(wrapper));

    // -------------------------------------------------------------- Add API for the "button_colour_flashing" blocks
    var wrapper = function(t1, t2, t3, t4)
    {
        t1 = t1 ? t1.toString() : '';
        t2 = t2 ? t2.toString() : '';
        t3 = t3 ? t3.toString() : '';
        t4 = t4 ? t4.toString() : '';
        var dummy = chrome.webview.hostObjects.sync.dotnet.SetValue('button_colour_flashing ' + t1 + ' ' + t2 + ' ' + t3 + ' ' + t4);
    };
    interpreter.setProperty(globalObject, 'button_colour_flashing',
                            interpreter.createNativeFunction(wrapper));

    // -------------------------------------------------------------- Add API for the "button_slot" blocks
    var wrapper = function(t1, t2)
    {
        t1 = t1 ? t1.toString() : '';
        t2 = t2 ? t2.toString() : '';
        var dummy = chrome.webview.hostObjects.sync.dotnet.SetValue('button_slot ' + t1 + ' ' + t2);
    };
    interpreter.setProperty(globalObject, 'button_slot',
                            interpreter.createNativeFunction(wrapper));

    // -------------------------------------------------------------- Add API for the "button_text" blocks
    var wrapper = function(t1, t2)
    {
        t1 = t1 ? t1.toString() : '';
        t2 = t2 ? t2.toString() : '';
        var dummy = chrome.webview.hostObjects.sync.dotnet.SetValue('button_text ' + t1 + ' ' + t2);
    };
    interpreter.setProperty(globalObject, 'button_text',
                            interpreter.createNativeFunction(wrapper));

    // =================================================
    //    CONTROLS
    // =================================================
    // -------------------------------------------------------------- Add API for the "open_textbox" blocks
    var wrapper = function()
    {
        var msg = chrome.webview.hostObjects.sync.dotnet.SetValue('open_textbox');
        return msg;
    };
    interpreter.setProperty(globalObject, 'open_textbox',
                            interpreter.createNativeFunction(wrapper));

    // -------------------------------------------------------------- Add API for the "close_textbox" blocks
    var wrapper = function()
    {
        var msg = chrome.webview.hostObjects.sync.dotnet.SetValue('close_textbox');
        return msg;
    };
    interpreter.setProperty(globalObject, 'close_textbox',
                            interpreter.createNativeFunction(wrapper));

    // -------------------------------------------------------------- Add API for the "clear_textbox" blocks
    var wrapper = function()
    {
        var msg = chrome.webview.hostObjects.sync.dotnet.SetValue('clear_textbox');
        return msg;
    };
    interpreter.setProperty(globalObject, 'clear_textbox',
                            interpreter.createNativeFunction(wrapper));

}



// ====================================================================================== 
//  CUSTOM BLOCKS
// ======================================================================================

Blockly.HSV_SATURATION = 0.5;// 0 (inclusive) to 1 (exclusive), defaulting to 0.45
Blockly.HSV_VALUE = 0.7; // 0 (inclusive) to 1 (exclusive), defaulting to 0.65

// ------------------------------------------------------------------ 
//  WAIT SECONDS
// ------------------------------------------------------------------
Blockly.Blocks['wait_seconds'] = 
{
    init: function() 
    {
        this.appendValueInput('SECONDS')
            .appendField('wait seconds');
        this.setInputsInline(true);
        this.setPreviousStatement(true, null);
        this.setNextStatement(true, null);
        this.setColour(120);
        this.setTooltip('Set the Text-To-Speech volume (0 to 100).');
        this.setHelpUrl('http://www.theremino.com');
    }
};
// ------------------------Generator function that calls "waitForSeconds"
Blockly.JavaScript['wait_seconds'] = function(block) 
{
    const seconds = Blockly.JavaScript.valueToCode(block, 'SECONDS', Blockly.JavaScript.ORDER_NONE) || "''";
    var code = 'waitForSeconds(' + seconds + ');\n';
    return code;
};

// ------------------------------------------------------------------ 
//   PRINT
// ------------------------------------------------------------------
Blockly.Blocks['print'] = 
{
    init: function() 
    {
        this.appendValueInput('TEXT')
            .appendField('print');
        this.setInputsInline(true);
        this.setPreviousStatement(true, null);
        this.setNextStatement(true, null);
        this.setColour(45);
        this.setTooltip('Print a text string or a number.');
        this.setHelpUrl('http://www.theremino.com');
    }
};
// -------------------------------------------- Generator function
Blockly.JavaScript['print'] = function(block) 
{
    //const msg = Blockly.JavaScript.valueToCode(block, 'TEXT', Blockly.JavaScript.ORDER_NONE) || "''";
    var msg = Blockly.JavaScript.valueToCode(block, 'TEXT', Blockly.JavaScript.ORDER_FUNCTION_CALL) || '\'\'';

    return 'print(' + msg + ');\n';
};

//------------------------------------------------------------------ 
//  BEEP
//------------------------------------------------------------------
Blockly.Blocks['beep'] = 
{
    init: function() 
    {
        this.appendValueInput('FREQ')
            .appendField('beep');
        this.appendValueInput('TIME')
            .appendField('');
        this.setInputsInline(true);
        this.setPreviousStatement(true, null);
        this.setNextStatement(true, null);
        this.setColour(45);
        this.setTooltip('Output a sound with frequency (Hz) and duration (mS).');
        this.setHelpUrl('http://www.theremino.com');
    }
};
// -------------------------------------------- Generator function
Blockly.JavaScript['beep'] = function(block) 
{
    const cmd = Blockly.JavaScript.valueToCode(block, 'FREQ', Blockly.JavaScript.ORDER_NONE) || "''";
    const txt = Blockly.JavaScript.valueToCode(block, 'TIME', Blockly.JavaScript.ORDER_NONE) || "''";
    return 'beep(' + cmd + ',' + txt + ');\n';
};


//------------------------------------------------------------------ 
//  READ-SLOT
//------------------------------------------------------------------
Blockly.Blocks['read_slot'] = 
{
    init: function() 
    {
        this.appendValueInput('SLOTNUM')
            .appendField('slot');
        this.setInputsInline(true);
        this.setOutput(true, 'Number');
        this.setColour(45);
        this.setTooltip('Returns numeric value from Slot.\n' +
                        'Slot numbers can range from 1 to 999');
        this.setHelpUrl('http://www.theremino.com');
    }
};
// -------------------------------------------- Generator function
Blockly.JavaScript['read_slot'] = function(block) 
{
    //var msg = Blockly.JavaScript.valueToCode(block, 'SLOTNUM', Blockly.JavaScript.ORDER_NONE) || "''";
    var msg = Blockly.JavaScript.valueToCode(block, 'SLOTNUM', Blockly.JavaScript.ORDER_FUNCTION_CALL) || '\'\'';

    let code = 'read_slot(' + msg + ')';
    return [code, Blockly.JavaScript.ORDER_FUNCTION_CALL];
};

//------------------------------------------------------------------ 
//  READ-SLOTTEXT
//------------------------------------------------------------------
Blockly.Blocks['read_slottext'] = 
{
    init: function() 
    {
        this.appendValueInput('SLOTNUM')
            .appendField('slot-text');
        this.setInputsInline(true);
        this.setOutput(true, 'String');
        this.setColour(45);
        this.setTooltip('Returns text string from a SlotText.\n' +
                        'SlotText numbers can range from 1 to 999');
        this.setHelpUrl('http://www.theremino.com');
    }
};
// -------------------------------------------- Generator function
Blockly.JavaScript['read_slottext'] = function(block) 
{
    //var msg = Blockly.JavaScript.valueToCode(block, 'SLOTNUM', Blockly.JavaScript.ORDER_NONE) || "''";
    var msg = Blockly.JavaScript.valueToCode(block, 'SLOTNUM', Blockly.JavaScript.ORDER_FUNCTION_CALL) || '\'\'';

    let code = 'read_slottext(' + msg + ')';
    return [code, Blockly.JavaScript.ORDER_FUNCTION_CALL];
};


//------------------------------------------------------------------ 
//  WRITE-SLOT
//------------------------------------------------------------------
Blockly.Blocks['write_slot'] = 
{
    init: function() 
    {
        this.appendValueInput('SLOTNUM')
            .appendField('slot');
        this.appendValueInput('VALUE')
            .appendField('=');
        this.setInputsInline(true);
        this.setPreviousStatement(true, null);
        this.setNextStatement(true, null);
        this.setColour(45);
        this.setTooltip('Write a numeric value to a Slot.\n' +
                        'Slot numbers can range from 1 to 999');
        this.setHelpUrl('http://www.theremino.com');
    }
};
// -------------------------------------------- Generator function
Blockly.JavaScript['write_slot'] = function(block) 
{
    const cmd = Blockly.JavaScript.valueToCode(block, 'SLOTNUM', Blockly.JavaScript.ORDER_NONE) || "''";
    const txt = Blockly.JavaScript.valueToCode(block, 'VALUE', Blockly.JavaScript.ORDER_NONE) || "''";
    return 'write_slot(' + cmd + ',' + txt + ');\n';
};


//------------------------------------------------------------------ 
//  WRITE-SLOTTEXT
//------------------------------------------------------------------
Blockly.Blocks['write_slottext'] = 
{
    init: function() 
    {
        this.appendValueInput('SLOTNUM')
            .appendField('slot-text');
        this.appendValueInput('TXT')
            .appendField('=');
        this.setInputsInline(true);
        this.setPreviousStatement(true, null);
        this.setNextStatement(true, null);
        this.setColour(45);
        this.setTooltip('Write text to a SlotText.\n' +
                        'SlotText numbers can range from 1 to 999');
        this.setHelpUrl('http://www.theremino.com');
    }
};
// -------------------------------------------- Generator function
Blockly.JavaScript['write_slottext'] = function(block) 
{
    const cmd = Blockly.JavaScript.valueToCode(block, 'SLOTNUM', Blockly.JavaScript.ORDER_NONE) || "''";
    const txt = Blockly.JavaScript.valueToCode(block, 'TXT', Blockly.JavaScript.ORDER_NONE) || "''";
    return 'write_slottext(' + cmd + ',' + txt + ');\n';
};


// ------------------------------------------------------------------ 
//   OPENAPP
// ------------------------------------------------------------------
Blockly.Blocks['open_app'] = 
{
    init: function() 
    {
        this.appendValueInput('TEXT')
            .appendField('open');
        this.setInputsInline(true);
        this.setPreviousStatement(true, null);
        this.setNextStatement(true, null);
        this.setColour(200);
        this.setTooltip('Open an app specified with a complete path \n' + 
                        'or with name and extension, if it is in the "Apps" or "Media" folders.\n \n' +
                        'This command can also open other file types, like midi and video files.');
        this.setHelpUrl('http://www.theremino.com');
    }
};
// -------------------------------------------- Generator function
Blockly.JavaScript['open_app'] = function(block) 
{
    //const msg = Blockly.JavaScript.valueToCode(block, 'TEXT', Blockly.JavaScript.ORDER_NONE) || "''";
    var msg = Blockly.JavaScript.valueToCode(block, 'TEXT', Blockly.JavaScript.ORDER_FUNCTION_CALL) || '\'\'';

    return 'open_app(' + msg + ');\n';
};

// ------------------------------------------------------------------ 
//   CLOSEAPP
// ------------------------------------------------------------------
Blockly.Blocks['close_app'] = 
{
    init: function() 
    {
        this.appendValueInput('TEXT')
            .appendField('close');
        this.setInputsInline(true);
        this.setPreviousStatement(true, null);
        this.setNextStatement(true, null);
        this.setColour(200);
        this.setTooltip('Close an app specified with a complete path \n' + 
                        'or with name and extension, if it is in the "Apps" or "Media" folders.\n' +
                        'This command can also close some window apps like "wmplayer.exe".');
        this.setHelpUrl('http://www.theremino.com');
    }
};
// -------------------------------------------- Generator function
Blockly.JavaScript['close_app'] = function(block) 
{
    //const msg = Blockly.JavaScript.valueToCode(block, 'TEXT', Blockly.JavaScript.ORDER_NONE) || "''";
    var msg = Blockly.JavaScript.valueToCode(block, 'TEXT', Blockly.JavaScript.ORDER_FUNCTION_CALL) || '\'\'';

    return 'close_app(' + msg + ');\n';
};


// ------------------------------------------------------------------ 
//   OPENAPPS
// ------------------------------------------------------------------
Blockly.Blocks['open_apps'] = 
{
    init: function() 
    {
        this.appendDummyInput()
            .appendField('open apps');
        this.setPreviousStatement(true, null);
        this.setNextStatement(true, null);
        this.setColour(200);
        this.setTooltip('Open all the "theremino_xxx.exe" apps that are in the "Apps" folder.');
        this.setHelpUrl('http://www.theremino.com');
    }
};
// -------------------------------------------- Generator function
Blockly.JavaScript['open_apps'] = function(block) 
{
    return 'open_apps();\n';
};

// ------------------------------------------------------------------ 
//   CLOSEAPPS
// ------------------------------------------------------------------
Blockly.Blocks['close_apps'] = 
{
    init: function() 
    {
        this.appendDummyInput()
            .appendField('close apps');
        this.setPreviousStatement(true, null);
        this.setNextStatement(true, null);
        this.setColour(200);
        this.setTooltip('Close all the previously opened applications.');
        this.setHelpUrl('http://www.theremino.com');
    }
};
// -------------------------------------------- Generator function
Blockly.JavaScript['close_apps'] = function(block) 
{
    return 'close_apps();\n';
};

// ------------------------------------------------------------------ 
//   CLOSEALL
// ------------------------------------------------------------------
Blockly.Blocks['close_all'] = 
{
    init: function() 
    {
        this.appendDummyInput()
            .appendField('close all');
        this.setPreviousStatement(true, null);
        this.setNextStatement(true, null);
        this.setColour(200);
        this.setTooltip('Close all the previously opened applications and Blockly too.');
        this.setHelpUrl('http://www.theremino.com');
    }
};
// -------------------------------------------- Generator function
Blockly.JavaScript['close_all'] = function(block) 
{
    return 'close_all();\n';
};



//------------------------------------------------------------------ 
//  TTS VOLUME
//------------------------------------------------------------------
Blockly.Blocks['tts_volume'] = 
{
    init: function() 
    {
        this.appendValueInput('VOLUME')
            .appendField('tts volume');
        this.setInputsInline(true);
        this.setPreviousStatement(true, null);
        this.setNextStatement(true, null);
        this.setColour(180);
        this.setTooltip('Set the Text-To-Speech volume (0 to 100).');
        this.setHelpUrl('http://www.theremino.com');
    }
};
// -------------------------------------------- Generator function
Blockly.JavaScript['tts_volume'] = function(block) 
{
    const cmd = Blockly.JavaScript.valueToCode(block, 'VOLUME', Blockly.JavaScript.ORDER_NONE) || "''";
    return 'tts_volume(' + cmd + ');\n';
};

//------------------------------------------------------------------ 
//  TTS SPEED
//------------------------------------------------------------------
Blockly.Blocks['tts_speed'] = 
{
    init: function() 
    {
        this.appendValueInput('SPEED')
            .appendField('tts speed');
        this.setInputsInline(true);
        this.setPreviousStatement(true, null);
        this.setNextStatement(true, null);
        this.setColour(180);
        this.setTooltip('Set the Text-To-Speech speed (normal = 0 / slow = -10 / fast = +10).');
        this.setHelpUrl('http://www.theremino.com');
    }
};
// -------------------------------------------- Generator function
Blockly.JavaScript['tts_speed'] = function(block) 
{
    const cmd = Blockly.JavaScript.valueToCode(block, 'SPEED', Blockly.JavaScript.ORDER_NONE) || "''";
    return 'tts_speed(' + cmd + ');\n';
};

//------------------------------------------------------------------ 
//  TTS VOICE
//------------------------------------------------------------------
Blockly.Blocks['tts_voice'] = 
{
    init: function() 
    {
        this.appendValueInput('VOICE')
            .appendField('tts voice');
        this.setInputsInline(true);
        this.setPreviousStatement(true, null);
        this.setNextStatement(true, null);
        this.setColour(180);
        this.setTooltip('Sets the Text-To-Speech voice (language ENG, ITA... and sex M or F).');
        this.setHelpUrl('http://www.theremino.com');
    }
};
// -------------------------------------------- Generator function
Blockly.JavaScript['tts_voice'] = function(block) 
{
    const cmd = Blockly.JavaScript.valueToCode(block, 'VOICE', Blockly.JavaScript.ORDER_NONE) || "''";
    return 'tts_voice(' + cmd + ');\n';
};

//------------------------------------------------------------------ 
//  TTS SPEAK
//------------------------------------------------------------------
Blockly.Blocks['tts_speak'] = 
{
    init: function() 
    {
        this.appendValueInput('TEXT')
            .appendField('tts speak');
        this.setInputsInline(true);
        this.setPreviousStatement(true, null);
        this.setNextStatement(true, null);
        this.setColour(180);
        this.setTooltip('Speak the text.');
        this.setHelpUrl('http://www.theremino.com');
    }
};
// -------------------------------------------- Generator function
Blockly.JavaScript['tts_speak'] = function(block) 
{
    const cmd = Blockly.JavaScript.valueToCode(block, 'TEXT', Blockly.JavaScript.ORDER_NONE) || "''";
    return 'tts_speak(' + cmd + ');\n';
};


// ------------------------------------------------------------------ 
//   TTS VOICES
// ------------------------------------------------------------------
Blockly.Blocks['tts_voices'] = 
{
    init: function() 
    {
        this.appendDummyInput()
            .appendField('tts voices');
        this.setOutput(true, 'String');
        this.setColour(180);
        this.setTooltip('Get the list of Text-To-Speech installed voices.');
        this.setHelpUrl('http://www.theremino.com');
    }
};
// -------------------------------------------- Generator function
Blockly.JavaScript['tts_voices'] = function(block) 
{
    let code = 'tts_voices()';
    return [code, Blockly.JavaScript.ORDER_FUNCTION_CALL];
};

// ------------------------------------------------------------------ 
//   TTS STOP
// ------------------------------------------------------------------
Blockly.Blocks['tts_stop'] = 
{
    init: function() 
    {
        this.appendDummyInput()
            .appendField('tts stop');
        this.setPreviousStatement(true, null);
        this.setNextStatement(true, null);
        this.setColour(180);
        this.setTooltip('Stop the Text-To-Speech speaking.');
        this.setHelpUrl('http://www.theremino.com');
    }
};
// -------------------------------------------- Generator function
Blockly.JavaScript['tts_stop'] = function(block) 
{
    return 'tts_stop();\n';
};

// ------------------------------------------------------------------ 
//   TTS PAUSE
// ------------------------------------------------------------------
Blockly.Blocks['tts_pause'] = 
{
    init: function() 
    {
        this.appendDummyInput()
            .appendField('tts pause');
        this.setPreviousStatement(true, null);
        this.setNextStatement(true, null);
        this.setColour(180);
        this.setTooltip('Pause the Text-To-Speech speaking.');
        this.setHelpUrl('http://www.theremino.com');
    }
};
// -------------------------------------------- Generator function
Blockly.JavaScript['tts_pause'] = function(block) 
{
    return 'tts_pause();\n';
};

// ------------------------------------------------------------------ 
//   TTS RESUME
// ------------------------------------------------------------------
Blockly.Blocks['tts_resume'] = 
{
    init: function() 
    {
        this.appendDummyInput()
            .appendField('tts resume');
        this.setPreviousStatement(true, null);
        this.setNextStatement(true, null);
        this.setColour(180);
        this.setTooltip('Resume the Text-To-Speech speaking after a pause command.');
        this.setHelpUrl('http://www.theremino.com');
    }
};
// -------------------------------------------- Generator function
Blockly.JavaScript['tts_resume'] = function(block) 
{
    return 'tts_resume();\n';
};

// ------------------------------------------------------------------ 
//   TTS WAIT
// ------------------------------------------------------------------
Blockly.Blocks['tts_wait'] = 
{
    init: function() 
    {
        this.appendDummyInput()
            .appendField('tts wait');
        this.setPreviousStatement(true, null);
        this.setNextStatement(true, null);
        this.setColour(180);
        this.setTooltip('Wait for the end of the speech.');
        this.setHelpUrl('http://www.theremino.com');
    }
};
// -------------------------------------------- Generator function
Blockly.JavaScript['tts_wait'] = function(block) 
{
    return 'tts_wait();\n';
};




// ------------------------------------------------------------------ 
//   COM PORT NAMES
// ------------------------------------------------------------------
Blockly.Blocks['com_portnames'] = 
{
    init: function() 
    {
        this.appendDummyInput()
            .appendField('com port names');
        this.setOutput(true, 'String');
        this.setColour(10);
        this.setTooltip('Get the COM port names.');
        this.setHelpUrl('http://www.theremino.com');
    }
};
// -------------------------------------------- Generator function
Blockly.JavaScript['com_portnames'] = function(block) 
{
    let code = 'com_portnames()';
    return [code, Blockly.JavaScript.ORDER_FUNCTION_CALL];
};		

//------------------------------------------------------------------ 
//  COM OPEN
//------------------------------------------------------------------
Blockly.Blocks['com_open'] = 
{
    init: function() 
    {
        this.appendValueInput('PORT')
            .appendField('com open port');
        this.appendValueInput('BAUD')
            .appendField('bauds');
        this.setInputsInline(true);
        this.setPreviousStatement(true, null);
        this.setNextStatement(true, null);
        this.setColour(10);
        this.setTooltip('Open a COM port with the specified Baud rate.');
        this.setHelpUrl('http://www.theremino.com');
    }
};
// -------------------------------------------- Generator function
Blockly.JavaScript['com_open'] = function(block) 
{
    const port = Blockly.JavaScript.valueToCode(block, 'PORT', Blockly.JavaScript.ORDER_NONE) || "''";
    const baud = Blockly.JavaScript.valueToCode(block, 'BAUD', Blockly.JavaScript.ORDER_NONE) || "''";
    return 'com_open(' + port + ',' + baud + ');\n';
};

// ------------------------------------------------------------------ 
//   COM CLOSE
// ------------------------------------------------------------------
Blockly.Blocks['com_close'] = 
{
    init: function() 
    {
        this.appendDummyInput()
            .appendField('com close');
        this.setPreviousStatement(true, null);
        this.setNextStatement(true, null);
        this.setColour(10);
        this.setTooltip('Close the COM port.');
        this.setHelpUrl('http://www.theremino.com');
    }
};
// -------------------------------------------- Generator function
Blockly.JavaScript['com_close'] = function(block) 
{
    return 'com_close();\n';
};

// ------------------------------------------------------------------ 
//   COM STATUS
// ------------------------------------------------------------------
Blockly.Blocks['com_status'] = 
{
    init: function() 
    {
        this.appendDummyInput()
            .appendField('com status');
        this.setOutput(true, 'String');
        this.setColour(10);
        this.setTooltip('Get the COM port status (OPENED or CLOSED).');
        this.setHelpUrl('http://www.theremino.com');
    }
};
// -------------------------------------------- Generator function
Blockly.JavaScript['com_status'] = function(block) 
{
    let code = 'com_status()';
    return [code, Blockly.JavaScript.ORDER_FUNCTION_CALL];
};

//------------------------------------------------------------------ 
//  COM SEND TEXT
//------------------------------------------------------------------
Blockly.Blocks['com_sendtext'] = 
{
    init: function() 
    {
        this.appendValueInput('TEXT')
            .appendField('com send text');
        this.setInputsInline(true);
        this.setPreviousStatement(true, null);
        this.setNextStatement(true, null);
        this.setColour(10);
        this.setTooltip('Send text to the COM port (use \\r and \\n for CR and LF)');
        this.setHelpUrl('http://www.theremino.com');
    }
};
// -------------------------------------------- Generator function
Blockly.JavaScript['com_sendtext'] = function(block) 
{
    const cmd = Blockly.JavaScript.valueToCode(block, 'TEXT', Blockly.JavaScript.ORDER_NONE) || "''";
    return 'com_sendtext(' + cmd + ');\n';
};


//------------------------------------------------------------------ 
//  COM SEND TEXT WITH TERMINATORS
//------------------------------------------------------------------
Blockly.Blocks['com_sendtextdouble'] = 
{
    init: function() 
    {
        this.appendValueInput('TEXT1')
            .appendField('com send text');
        this.appendValueInput('TEXT2')
            .appendField('and');
        this.setInputsInline(true);
        this.setPreviousStatement(true, null);
        this.setNextStatement(true, null);
        this.setColour(10);
        this.setTooltip('Send text to the COM port (use \\r and \\n for CR and LF)');
        this.setHelpUrl('http://www.theremino.com');
    }
};
// -------------------------------------------- Generator function
Blockly.JavaScript['com_sendtextdouble'] = function(block) 
{
    const text1 = Blockly.JavaScript.valueToCode(block, 'TEXT1', Blockly.JavaScript.ORDER_NONE) || "''";
    const text2 = Blockly.JavaScript.valueToCode(block, 'TEXT2', Blockly.JavaScript.ORDER_NONE) || "''";
    return 'com_sendtextdouble(' + text1 + ',' + text2 + ');\n';
};

// ------------------------------------------------------------------ 
//   COM RECEIVED
// ------------------------------------------------------------------
Blockly.Blocks['com_received'] = 
{
    init: function() 
    {
        this.appendDummyInput()
            .appendField('com received');
        this.setOutput(true, 'String');
        this.setColour(10);
        this.setTooltip('Get the COM port received buffer.');
        this.setHelpUrl('http://www.theremino.com');
    }
};
// -------------------------------------------- Generator function
Blockly.JavaScript['com_received'] = function(block) 
{
    let code = 'com_received()';
    return [code, Blockly.JavaScript.ORDER_FUNCTION_CALL];
};


// ------------------------------------------------------------------ 
//   COM RECEIVEUNTIL
// ------------------------------------------------------------------
Blockly.Blocks['com_receiveuntil'] = 
{
    init: function() 
    {
        this.appendValueInput('TEXT')
            .appendField('com receive until');
        this.setInputsInline(true);
        this.setOutput(true, 'String');
        this.setColour(10);
        this.setTooltip('Get the COM port received buffer when it contains the required string.');
        this.setHelpUrl('http://www.theremino.com');
    }
};
// -------------------------------------------- Generator function
Blockly.JavaScript['com_receiveuntil'] = function(block) 
{
    var msg = Blockly.JavaScript.valueToCode(block, 'TEXT', Blockly.JavaScript.ORDER_FUNCTION_CALL) || '\'\'';
    let code = 'com_receiveuntil(' + msg + ')';
    return [code, Blockly.JavaScript.ORDER_FUNCTION_CALL];
};


// ------------------------------------------------------------------ 
//   COM RECEIVE NCHARS
// ------------------------------------------------------------------
Blockly.Blocks['com_receivenchars'] = 
{
    init: function() 
    {
        this.appendValueInput('NCHARS')
            .appendField('com receive nchars');
        this.setInputsInline(true);
        this.setOutput(true, 'String');
        this.setColour(10);
        this.setTooltip('Get the COM port received buffer when it contains the required number of characters.');
        this.setHelpUrl('http://www.theremino.com');
    }
};
// -------------------------------------------- Generator function
Blockly.JavaScript['com_receivenchars'] = function(block) 
{
    var msg = Blockly.JavaScript.valueToCode(block, 'NCHARS', Blockly.JavaScript.ORDER_FUNCTION_CALL) || '\'\'';
    let code = 'com_receivenchars(' + msg + ')';
    return [code, Blockly.JavaScript.ORDER_FUNCTION_CALL];
};


// ------------------------------------------------------------------ 
//   COM DISCARD IN-BUFFER
// ------------------------------------------------------------------
Blockly.Blocks['com_discardinbuffer'] = 
{
    init: function() 
    {
        this.appendDummyInput()
            .appendField('com discard in-buffer');
        this.setPreviousStatement(true, null);
        this.setNextStatement(true, null);
        this.setColour(10);
        this.setTooltip('Empty the COM port input buffer.');
        this.setHelpUrl('http://www.theremino.com');
    }
};
// -------------------------------------------- Generator function
Blockly.JavaScript['com_discardinbuffer'] = function(block) 
{
    return 'com_discardinbuffer();\n';
};

// ------------------------------------------------------------------ 
//   COM DISCARD OUT-BUFFER
// ------------------------------------------------------------------
Blockly.Blocks['com_discardoutbuffer'] = 
{
    init: function() 
    {
        this.appendDummyInput()
            .appendField('com discard out-buffer');
        this.setPreviousStatement(true, null);
        this.setNextStatement(true, null);
        this.setColour(10);
        this.setTooltip('Empty the COM port output buffer.');
        this.setHelpUrl('http://www.theremino.com');
    }
};
// -------------------------------------------- Generator function
Blockly.JavaScript['com_discardoutbuffer'] = function(block) 
{
    return 'com_discardoutbuffer();\n';
};

// ------------------------------------------------------------------ 
//   ENABLE DTR
// ------------------------------------------------------------------
Blockly.Blocks['com_enabledtr'] = 
{
    init: function() 
    {
        this.appendDummyInput()
            .appendField('com enable dtr');
        this.setPreviousStatement(true, null);
        this.setNextStatement(true, null);
        this.setColour(10);
        this.setTooltip('Enable the COM port DTR signal.');
        this.setHelpUrl('http://www.theremino.com');
    }
};
// -------------------------------------------- Generator function
Blockly.JavaScript['com_enabledtr'] = function(block) 
{
    return 'com_enabledtr();\n';
};

// ------------------------------------------------------------------ 
//   ENABLE RTS
// ------------------------------------------------------------------
Blockly.Blocks['com_enablerts'] = 
{
    init: function() 
    {
        this.appendDummyInput()
            .appendField('com enable rts');
        this.setPreviousStatement(true, null);
        this.setNextStatement(true, null);
        this.setColour(10);
        this.setTooltip('Enable the COM port RTS signal.');
        this.setHelpUrl('http://www.theremino.com');
    }
};
// -------------------------------------------- Generator function
Blockly.JavaScript['com_enablerts'] = function(block) 
{
    return 'com_enablerts();\n';
};

// ------------------------------------------------------------------ 
//   DISABLE DTR
// ------------------------------------------------------------------
Blockly.Blocks['com_disabledtr'] = 
{
    init: function() 
    {
        this.appendDummyInput()
            .appendField('com disable dtr');
        this.setPreviousStatement(true, null);
        this.setNextStatement(true, null);
        this.setColour(10);
        this.setTooltip('Disable the COM port DTR signal.');
        this.setHelpUrl('http://www.theremino.com');
    }
};
// -------------------------------------------- Generator function
Blockly.JavaScript['com_disabledtr'] = function(block) 
{
    return 'com_disabledtr();\n';
};

// ------------------------------------------------------------------ 
//   DISABLE RTS
// ------------------------------------------------------------------
Blockly.Blocks['com_disablerts'] = 
{
    init: function() 
    {
        this.appendDummyInput()
            .appendField('com disable rts');
        this.setPreviousStatement(true, null);
        this.setNextStatement(true, null);
        this.setColour(10);
        this.setTooltip('Disable the COM port RTS signal.');
        this.setHelpUrl('http://www.theremino.com');
    }
};
// -------------------------------------------- Generator function
Blockly.JavaScript['com_disablerts'] = function(block) 
{
    return 'com_disablerts();\n';
};

// ------------------------------------------------------------------ 
//   COM GET DSR
// ------------------------------------------------------------------
Blockly.Blocks['com_getdsr'] = 
{
    init: function() 
    {
        this.appendDummyInput()
            .appendField('com get DSR');
        this.setOutput(true, 'String');
        this.setColour(10);
        this.setTooltip('Get the COM port DSR signal status (ON / OFF).');
        this.setHelpUrl('http://www.theremino.com');
    }
};
// -------------------------------------------- Generator function
Blockly.JavaScript['com_getdsr'] = function(block) 
{
    let code = 'com_getdsr()';
    return [code, Blockly.JavaScript.ORDER_FUNCTION_CALL];
};		

// ------------------------------------------------------------------ 
//   COM GET CTS
// ------------------------------------------------------------------
Blockly.Blocks['com_getcts'] = 
{
    init: function() 
    {
        this.appendDummyInput()
            .appendField('com get CTS');
        this.setOutput(true, 'String');
        this.setColour(10);
        this.setTooltip('Get the COM port CTS signal status (ON / OFF).');
        this.setHelpUrl('http://www.theremino.com');
    }
};
// -------------------------------------------- Generator function
Blockly.JavaScript['com_getcts'] = function(block) 
{
    let code = 'com_getcts()';
    return [code, Blockly.JavaScript.ORDER_FUNCTION_CALL];
};		




// ------------------------------------------------------------------ 
//   BUTTONS - ENABLE ALL BUTTONS
// ------------------------------------------------------------------
Blockly.Blocks['enable_all_buttons'] = 
{
    init: function() 
    {
        this.appendDummyInput()
            .appendField('enable all buttons');
        this.setPreviousStatement(true, null);
        this.setNextStatement(true, null);
        this.setColour(60);
        this.setTooltip('Enable all the buttons.');
        this.setHelpUrl('http://www.theremino.com');
    }
};
// -------------------------------------------- Generator function
Blockly.JavaScript['enable_all_buttons'] = function(block) 
{
    return 'enable_all_buttons();\n';
};

// ------------------------------------------------------------------ 
//   BUTTONS - DISABLE ALL BUTTONS
// ------------------------------------------------------------------
Blockly.Blocks['disable_all_buttons'] = 
{
    init: function() 
    {
        this.appendDummyInput()
            .appendField('disable all buttons');
        this.setPreviousStatement(true, null);
        this.setNextStatement(true, null);
        this.setColour(60);
        this.setTooltip('disable all the buttons.');
        this.setHelpUrl('http://www.theremino.com');
    }
};
// -------------------------------------------- Generator function
Blockly.JavaScript['disable_all_buttons'] = function(block) 
{
    return 'disable_all_buttons();\n';
};


//------------------------------------------------------------------ 
//  BUTTONS - ENABLE BUTTON
//------------------------------------------------------------------
Blockly.Blocks['enable_button'] = 
{
    init: function() 
    {
        this.appendValueInput('TEXT')
            .appendField('enable button');
        this.setInputsInline(true);
        this.setPreviousStatement(true, null);
        this.setNextStatement(true, null);
        this.setColour(60);
        this.setTooltip('Enable the specified button.');
        this.setHelpUrl('http://www.theremino.com');
    }
};
// -------------------------------------------- Generator function
Blockly.JavaScript['enable_button'] = function(block) 
{
    const cmd = Blockly.JavaScript.valueToCode(block, 'TEXT', Blockly.JavaScript.ORDER_NONE) || "''";
    return 'enable_button(' + cmd + ');\n';
};

//------------------------------------------------------------------ 
//  BUTTONS - DISABLE BUTTON
//------------------------------------------------------------------
Blockly.Blocks['disable_button'] = 
{
    init: function() 
    {
        this.appendValueInput('TEXT')
            .appendField('disable button');
        this.setInputsInline(true);
        this.setPreviousStatement(true, null);
        this.setNextStatement(true, null);
        this.setColour(60);
        this.setTooltip('Disable the specified button.');
        this.setHelpUrl('http://www.theremino.com');
    }
};
// -------------------------------------------- Generator function
Blockly.JavaScript['disable_button'] = function(block) 
{
    const cmd = Blockly.JavaScript.valueToCode(block, 'TEXT', Blockly.JavaScript.ORDER_NONE) || "''";
    return 'disable_button(' + cmd + ');\n';
};

//------------------------------------------------------------------ 
//  BUTTON COLOUR
//------------------------------------------------------------------
Blockly.Blocks['button_colour'] = 
{
    init: function() 
    {
        this.appendValueInput('TEXT')
            .appendField('button');
        this.appendValueInput('COLOUR')
            .appendField('colour');
        this.setInputsInline(true);
        this.setPreviousStatement(true, null);
        this.setNextStatement(true, null);
        this.setColour(60);
        this.setTooltip('Set the specified button colour.');
        this.setHelpUrl('http://www.theremino.com');
    }
};
// -------------------------------------------- Generator function
Blockly.JavaScript['button_colour'] = function(block) 
{
    const num = Blockly.JavaScript.valueToCode(block, 'TEXT', Blockly.JavaScript.ORDER_NONE) || "''";
    const col = Blockly.JavaScript.valueToCode(block, 'COLOUR', Blockly.JavaScript.ORDER_NONE) || "''";
    return 'button_colour(' + num + ',' + col + ');\n';
};

//------------------------------------------------------------------ 
//  BUTTON COLOUR FLASHING
//------------------------------------------------------------------
Blockly.Blocks['button_colour_flashing'] = 
{
    init: function() 
    {
        this.appendValueInput('TEXT')
            .appendField('button');
        this.appendValueInput('COLOUR1')
            .appendField('col1');
        this.appendValueInput('COLOUR2')
            .appendField('col2');
        this.appendValueInput('SPEED')
            .appendField('speed');
        this.setInputsInline(true);
        this.setPreviousStatement(true, null);
        this.setNextStatement(true, null);
        this.setColour(60);
        this.setTooltip('Set the specified button flashing colours and speed (0 = medium speed / 1 to 9 = slow to fast)');
        this.setHelpUrl('http://www.theremino.com');
    }
};
// -------------------------------------------- Generator function
Blockly.JavaScript['button_colour_flashing'] = function(block) 
{
    const num = Blockly.JavaScript.valueToCode(block, 'TEXT', Blockly.JavaScript.ORDER_NONE) || "''";
    const col1 = Blockly.JavaScript.valueToCode(block, 'COLOUR1', Blockly.JavaScript.ORDER_NONE) || "''";
    const col2 = Blockly.JavaScript.valueToCode(block, 'COLOUR2', Blockly.JavaScript.ORDER_NONE) || "''";
    const speed = Blockly.JavaScript.valueToCode(block, 'SPEED', Blockly.JavaScript.ORDER_NONE) || "''";

    return 'button_colour_flashing(' + num + ',' + col1 + ',' + col2 + ',' + speed + ');\n';
};

//------------------------------------------------------------------ 
//  BUTTON SLOT
//------------------------------------------------------------------
Blockly.Blocks['button_slot'] = 
{
    init: function() 
    {
        this.appendValueInput('TEXT')
            .appendField('button');
        this.appendValueInput('SLOT')
            .appendField('slot');
        this.setInputsInline(true);
        this.setPreviousStatement(true, null);
        this.setNextStatement(true, null);
        this.setColour(60);
        this.setTooltip('Set the Slot to activate the specified button.');
        this.setHelpUrl('http://www.theremino.com');
    }
};
// -------------------------------------------- Generator function
Blockly.JavaScript['button_slot'] = function(block) 
{
    const num = Blockly.JavaScript.valueToCode(block, 'TEXT', Blockly.JavaScript.ORDER_NONE) || "''";
    const slot = Blockly.JavaScript.valueToCode(block, 'SLOT', Blockly.JavaScript.ORDER_NONE) || "''";
    return 'button_slot(' + num + ',' + slot + ');\n';
};

//------------------------------------------------------------------ 
//  BUTTON TEXT
//------------------------------------------------------------------
Blockly.Blocks['button_text'] = 
{
    init: function() 
    {
        this.appendValueInput('TEXT')
            .appendField('button');
        this.appendValueInput('BUTTONTEXT')
            .appendField('text');
        this.setInputsInline(true);
        this.setPreviousStatement(true, null);
        this.setNextStatement(true, null);
        this.setColour(60);
        this.setTooltip('Set the text to visualize on the specified button.');
        this.setHelpUrl('http://www.theremino.com');
    }
};
// -------------------------------------------- Generator function
Blockly.JavaScript['button_text'] = function(block) 
{
    const num = Blockly.JavaScript.valueToCode(block, 'TEXT', Blockly.JavaScript.ORDER_NONE) || "''";
    const text = Blockly.JavaScript.valueToCode(block, 'BUTTONTEXT', Blockly.JavaScript.ORDER_NONE) || "''";
    return 'button_text(' + num + ',' + text + ');\n';
};


// ------------------------------------------------------------------ 
//   CONTROLS - OPEN TEXTBOX
// ------------------------------------------------------------------
Blockly.Blocks['open_textbox'] = 
{
    init: function() 
    {
        this.appendDummyInput()
            .appendField('open textbox');
        this.setPreviousStatement(true, null);
        this.setNextStatement(true, null);
        this.setColour(60);
        this.setTooltip('Open the textbox for Print and Debug.');
        this.setHelpUrl('http://www.theremino.com');
    }
};
// -------------------------------------------- Generator function
Blockly.JavaScript['open_textbox'] = function(block) 
{
    return 'open_textbox();\n';
};

// ------------------------------------------------------------------ 
//   CONTROLS - CLOSE TEXTBOX
// ------------------------------------------------------------------
Blockly.Blocks['close_textbox'] = 
{
    init: function() 
    {
        this.appendDummyInput()
            .appendField('close textbox');
        this.setPreviousStatement(true, null);
        this.setNextStatement(true, null);
        this.setColour(60);
        this.setTooltip('Close the Print and Debug textbox.');
        this.setHelpUrl('http://www.theremino.com');
    }
};
// -------------------------------------------- Generator function
Blockly.JavaScript['close_textbox'] = function(block) 
{
    return 'close_textbox();\n';
};

// ------------------------------------------------------------------ 
//   CONTROLS - CLEAR TEXTBOX
// ------------------------------------------------------------------
Blockly.Blocks['clear_textbox'] = 
{
    init: function() 
    {
        this.appendDummyInput()
            .appendField('clear textbox');
        this.setPreviousStatement(true, null);
        this.setNextStatement(true, null);
        this.setColour(60);
        this.setTooltip('Clear all the text in the Print and Debug textbox.');
        this.setHelpUrl('http://www.theremino.com');
    }
};
// -------------------------------------------- Generator function
Blockly.JavaScript['clear_textbox'] = function(block) 
{
    return 'clear_textbox();\n';
};



// ====================================================================================== 
//  FUNCTIONS
// ======================================================================================

// -------------------------------------------------------------------------------------------------
//  THIS IS A SUBSTITUTE FOR TEXT-REPLACE THAT CRASHES THE JS-INTERPRETER
// -------------------------------------------------------------------------------------------------
Blockly.JavaScript['text_replace'] = function(block) 
{
    const text = Blockly.JavaScript.valueToCode(block, 'TEXT', Blockly.JavaScript.ORDER_NONE) || "''";
    const from = Blockly.JavaScript.valueToCode(block, 'FROM', Blockly.JavaScript.ORDER_NONE) || "''";
    const to = Blockly.JavaScript.valueToCode(block, 'TO', Blockly.JavaScript.ORDER_NONE) || "''";
    const functionName = Blockly.JavaScript.provideFunction_
    ('textReplace', `function ${Blockly.JavaScript.FUNCTION_NAME_PLACEHOLDER_}(text, from, to) {
  return text.replace(from, to); 
}`
    );
    const code = functionName + '(' + text + ', ' + from + ', ' + to + ')';
    return [code, Blockly.JavaScript.ORDER_FUNCTION_CALL];
};




